@@ -76,11 +76,9 @@ class AggregateDeclaration : public ScopeDsymbol
 public:
     Type *type;
     StorageClass storage_class;
-    Prot protection;
     unsigned structsize;        // size of struct
     unsigned alignsize;         // size of struct for alignment purposes
     VarDeclarations fields;     // VarDeclaration fields
-    Sizeok sizeok;              // set when structsize contains valid data
     Dsymbol *deferred;          // any deferred semantic2() or semantic3() symbol
 
     ClassKind::Type classKind;  // specifies the linkage type
@@ -108,7 +106,6 @@ public:
     CtorDeclaration *defaultCtor;
 
     AliasThis *aliasthis;       // forward unresolved lookups to aliasthis
-    bool noDefaultCtor;         // no default construction
 
     DtorDeclarations dtors;     // Array of destructors
     DtorDeclaration *dtor;      // aggregate destructor
@@ -118,6 +115,10 @@ public:
 
     Expression *getRTInfo;      // pointer to GC info generated by object.RTInfo(this)
 
+    Prot protection;
+    bool noDefaultCtor;         // no default construction
+    Sizeok sizeok;              // set when structsize contains valid data
+
     virtual Scope *newScope(Scope *sc);
     void setScope(Scope *sc);
     bool determineFields();
@@ -138,9 +139,11 @@ public:
     // 'this' type
     Type *handleType() { return type; }
 
+#if !IN_LLVM
     // Back end
     Symbol *stag;               // tag symbol for debug data
     Symbol *sinit;
+#endif
 
     AggregateDeclaration *isAggregateDeclaration() { return this; }
     void accept(Visitor *v) { v->visit(this); }
@@ -160,13 +163,18 @@ class StructDeclaration : public AggregateDeclaration
 public:
     bool zeroInit;              // !=0 if initialize with 0 fill
     bool hasIdentityAssign;     // true if has identity opAssign
+    bool hasBlitAssign;         // true if opAssign is a blit
     bool hasIdentityEquals;     // true if has identity opEquals
     bool hasNoFields;           // has no fields
+    bool hasCopyCtor;           // copy constructor
+    // Even if struct is defined as non-root symbol, some built-in operations
+    // (e.g. TypeidExp, NewExp, ArrayLiteralExp, etc) request its TypeInfo.
+    // For those, today TypeInfo_Struct is generated in COMDAT.
+    bool requestTypeInfo;
+
     FuncDeclarations postblits; // Array of postblit functions
     FuncDeclaration *postblit;  // aggregate postblit
 
-    bool hasCopyCtor;           // copy constructor
-
     FuncDeclaration *xeq;       // TypeInfo_Struct.xopEquals
     FuncDeclaration *xcmp;      // TypeInfo_Struct.xopCmp
     FuncDeclaration *xhash;     // TypeInfo_Struct.xtoHash
@@ -176,14 +184,8 @@ public:
     structalign_t alignment;    // alignment applied outside of the struct
     StructPOD ispod;            // if struct is POD
 
-    // For 64 bit Efl function call/return ABI
-    Type *arg1type;
-    Type *arg2type;
-
-    // Even if struct is defined as non-root symbol, some built-in operations
-    // (e.g. TypeidExp, NewExp, ArrayLiteralExp, etc) request its TypeInfo.
-    // For those, today TypeInfo_Struct is generated in COMDAT.
-    bool requestTypeInfo;
+    // ABI-specific type(s) if the struct can be passed in registers
+    TypeTuple *argTypes;
 
     static StructDeclaration *create(Loc loc, Identifier *id, bool inObject);
     Dsymbol *syntaxCopy(Dsymbol *s);
@@ -196,6 +198,9 @@ public:
 
     StructDeclaration *isStructDeclaration() { return this; }
     void accept(Visitor *v) { v->visit(this); }
+
+    unsigned numArgTypes() const;
+    Type *argType(unsigned index);
 };
 
 class UnionDeclaration : public StructDeclaration
@@ -270,13 +275,13 @@ public:
     bool stack;                         // true if this is a scope class
     int cppDtorVtblIndex;               // slot reserved for the virtual destructor [extern(C++)]
     bool inuse;                         // to prevent recursive attempts
-    bool isActuallyAnonymous;           // true if this class has an identifier, but was originally declared anonymous
-                                        // used in support of https://issues.dlang.org/show_bug.cgi?id=17371
 
     Abstract isabstract;                // 0: fwdref, 1: is abstract class, 2: not abstract
     Baseok baseok;                      // set the progress of base classes resolving
     ObjcClassDeclaration objc;          // Data for a class declaration that is needed for the Objective-C integration
+#if !IN_LLVM
     Symbol *cpp_type_info_ptr_sym;      // cached instance of class Id.cpp_type_info_ptr
+#endif
 
     static ClassDeclaration *create(Loc loc, Identifier *id, BaseClasses *baseclasses, Dsymbols *members, bool inObject);
     Dsymbol *syntaxCopy(Dsymbol *s);
@@ -286,7 +291,6 @@ public:
     #define OFFSET_RUNTIME 0x76543210
     #define OFFSET_FWDREF 0x76543211
     virtual bool isBaseOf(ClassDeclaration *cd, int *poffset);
-    bool isAnonymous();
 
     bool isBaseInfoComplete();
     Dsymbol *search(const Loc &loc, Identifier *ident, int flags = SearchLocalsOnly);
