@@ -41,7 +41,7 @@ class StringExp;
 struct UnionExp;
 #ifdef IN_GCC
 typedef union tree_node Symbol;
-#else
+#elif !IN_LLVM
 struct Symbol;          // back end symbol
 #endif
 
@@ -233,6 +233,7 @@ public:
     ModuleInitExp* isModuleInitExp();
     FuncInitExp* isFuncInitExp();
     PrettyFuncInitExp* isPrettyFuncInitExp();
+    ClassReferenceExp* isClassReferenceExp();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -352,8 +353,6 @@ public:
 class NullExp : public Expression
 {
 public:
-    unsigned char committed;    // !=0 if type is committed
-
     bool equals(const RootObject *o) const;
     bool isBool(bool result);
     StringExp *toStringExp();
@@ -390,6 +389,11 @@ public:
         assert(sz == 1);
         return {len, static_cast<const char *>(string)};
     }
+    // ditto
+    DArray<const unsigned char> peekData() const
+    {
+        return {len * sz, static_cast<const unsigned char *>(string)};
+    }
 #endif
     size_t numberOfCodeUnits(int tynto = 0) const;
     void writeTo(void* dest, bool zero, int tyto = 0) const;
@@ -463,10 +467,10 @@ public:
     // now contain pointers to themselves. While in toElem, contains a pointer
     // to the memory used to build the literal for resolving such references.
     llvm::Value *inProgressMemory;
+#else
+    Symbol *sym;        // back end symbol to initialize with literal
 #endif
 
-    Symbol *sym;                // back end symbol to initialize with literal
-
     /** pointer to the origin instance of the expression.
      * once a new expression is created, origin is set to 'this'.
      * anytime when an expression copy is created, 'origin' pointer is set to
@@ -572,8 +576,8 @@ class SymbolExp : public Expression
 {
 public:
     Declaration *var;
-    bool hasOverloads;
     Dsymbol *originalScope;
+    bool hasOverloads;
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -827,6 +831,7 @@ public:
     Expressions *arguments;     // function arguments
     FuncDeclaration *f;         // symbol to call
     bool directcall;            // true if a virtual call is devirtualized
+    bool inDebugStatement;      // true if this was in a debug statement
     VarDeclaration *vthis2;     // container for multi-context
 
     static CallExp *create(Loc loc, Expression *e, Expressions *exps);
@@ -1049,8 +1054,9 @@ public:
     void accept(Visitor *v) { v->visit(this); }
 };
 
-enum MemorySet
+enum class MemorySet
 {
+    none            = 0,    // simple assignment
     blockAssign     = 1,    // setting the contents of an array
     referenceInit   = 2     // setting the reference of STCref variable
 };
@@ -1058,7 +1064,7 @@ enum MemorySet
 class AssignExp : public BinExp
 {
 public:
-    int memset;         // combination of MemorySet flags
+    MemorySet memset;
 
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *ex);
@@ -1299,8 +1305,6 @@ public:
 class DefaultInitExp : public Expression
 {
 public:
-    TOK subop;             // which of the derived classes this is
-
     void accept(Visitor *v) { v->visit(this); }
 };
 
