@@ -41,6 +41,7 @@ AddrExp *createAddrExp(const Loc &, Expression *);
 DsymbolExp *createDsymbolExp(const Loc &, Dsymbol *, bool = false);
 Expression *createExpression(const Loc &loc, TOK op, int size);
 InlineAsmStatement *createInlineAsmStatement(const Loc &loc, Token *tokens);
+GccAsmStatement *createGccAsmStatement(const Loc &loc, Token *tokens);
 TypeDelegate *createTypeDelegate(Type *t);
 TypeIdentifier *createTypeIdentifier(const Loc &loc, Identifier *ident);
 
@@ -50,8 +51,8 @@ Strings *createStrings();
 //Loc createLoc(const char * filename, uint linnum, uint charnum);
 
 /*
- * Define bindD<Type>::create(...) templated functions, to create D objects in templated code (class type is template parameter).
- * Used e.g. in toir.cpp
+ * Define bindD<Type>::create(...) templated functions, to create D objects in
+ * templated code (class type is template parameter). Used e.g. in toir.cpp
  */
 template <class T> struct bindD {
   template <typename... Args> T *create(Args...) {
