@@ -1,6 +1,6 @@
 
 /* Compiler implementation of the D programming language
- * Copyright (C) 1999-2019 by The D Language Foundation, All Rights Reserved
+ * Copyright (C) 1999-2020 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
  * http://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
@@ -22,6 +22,7 @@ public:
     const char *toChars() const;
     void addMember(Scope *sc, ScopeDsymbol *sds);
     const char *kind() const;
+    DebugSymbol *isDebugSymbol();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -35,5 +36,6 @@ public:
     const char *toChars() const;
     void addMember(Scope *sc, ScopeDsymbol *sds);
     const char *kind() const;
+    VersionSymbol *isVersionSymbol();
     void accept(Visitor *v) { v->visit(this); }
 };
