@@ -37,13 +37,8 @@
 #include "gen/logger.h"
 #include "gen/optimizer.h"
 
-#if LDC_LLVM_VER >= 400
 #include "llvm/Bitcode/BitcodeWriter.h"
 #include "llvm/Support/Chrono.h"
-#else
-#include "llvm/Bitcode/ReaderWriter.h"
-#include "llvm/Support/TimeValue.h"
-#endif
 #include "llvm/Support/FileSystem.h"
 #include "llvm/Support/MD5.h"
 #include "llvm/Support/Path.h"
@@ -70,13 +65,22 @@ static bool createSymLink(const char *to, const char *from) {
 #include <windows.h>
 namespace llvm {
 namespace sys {
+#if LDC_LLVM_VER >= 1100
+namespace windows {
+// Fwd declaration to an internal LLVM function.
+std::error_code widenPath(const llvm::Twine &Path8,
+                          llvm::SmallVectorImpl<wchar_t> &Path16,
+                          size_t MaxPathLen = MAX_PATH);
+}
+#else
 namespace path {
 // Fwd declaration to an internal LLVM function.
 std::error_code widenPath(const llvm::Twine &Path8,
                           llvm::SmallVectorImpl<wchar_t> &Path16);
 }
-}
-}
+#endif // LDC_LLVM_VER < 1100
+} // namespace sys
+} // namespace llvm
 // Returns true upon error.
 namespace {
 template <typename FType>
@@ -88,11 +92,17 @@ bool createLink(FType f, const char *to, const char *from) {
   //
   //===----------------------------------------------------------------------===//
 
+#if LDC_LLVM_VER >= 1100
+  using llvm::sys::windows::widenPath;
+#else
+  using llvm::sys::path::widenPath;
+#endif
+
   llvm::SmallVector<wchar_t, 128> wide_from;
   llvm::SmallVector<wchar_t, 128> wide_to;
-  if (llvm::sys::path::widenPath(from, wide_from))
+  if (widenPath(from, wide_from))
     return true;
-  if (llvm::sys::path::widenPath(to, wide_to))
+  if (widenPath(to, wide_to))
     return true;
 
   if (!(*f)(wide_from.begin(), wide_to.begin(), NULL))
@@ -144,7 +154,7 @@ llvm::cl::opt<RetrievalMode> cacheRecoveryMode(
     "cache-retrieval", llvm::cl::ZeroOrMore,
     llvm::cl::desc("Set the cache retrieval mechanism (default: copy)."),
     llvm::cl::init(RetrievalMode::Copy),
-    clEnumValues(
+    llvm::cl::values(
         clEnumValN(RetrievalMode::Copy, "copy",
                    "Make a copy of the cache file"),
         clEnumValN(RetrievalMode::HardLink, "hardlink",
@@ -169,14 +179,10 @@ bool isPruningEnabled() {
   return false;
 }
 
-#if LDC_LLVM_VER >= 400
 llvm::sys::TimePoint<std::chrono::seconds> getTimeNow() {
   using namespace std::chrono;
   return time_point_cast<seconds>(system_clock::now());
 }
-#else
-llvm::sys::TimeValue getTimeNow() { return llvm::sys::TimeValue::now(); }
-#endif
 
 /// A raw_ostream that creates a hash of what is written to it.
 /// This class does not encounter output errors.
@@ -309,13 +315,9 @@ void outputIR2ObjRelevantCmdlineArgs(llvm::raw_ostream &hash_os) {
   const auto relocModel = opts::getRelocModel();
   if (relocModel.hasValue())
     hash_os << relocModel.getValue();
-#if LDC_LLVM_VER >= 600
   const auto codeModel = opts::getCodeModel();
   if (codeModel.hasValue())
     hash_os << codeModel.getValue();
-#else
-  hash_os << opts::getCodeModel();
-#endif
 #if LDC_LLVM_VER >= 800
   const auto framePointerUsage = opts::framePointerUsage();
   if (framePointerUsage.hasValue())
