@@ -11,19 +11,21 @@
 
 // Pull in command-line options and helper functions from special LLVM header
 // shared by multiple LLVM tools.
-#if LDC_LLVM_VER >= 700
-#include "llvm/CodeGen/CommandFlags.inc"
-#elif LDC_LLVM_VER >= 600
-#include "llvm/CodeGen/CommandFlags.def"
-#else
+#if LDC_LLVM_VER >= 1100
 #include "llvm/CodeGen/CommandFlags.h"
+static llvm::codegen::RegisterCodeGenFlags CGF;
+using namespace llvm;
+#elif LDC_LLVM_VER >= 700
+#include "llvm/CodeGen/CommandFlags.inc"
+#else
+#include "llvm/CodeGen/CommandFlags.def"
 #endif
 
 static cl::opt<bool>
     DisableRedZone("disable-red-zone", cl::ZeroOrMore,
                    cl::desc("Do not emit code that uses the red zone."));
 
-#if LDC_LLVM_VER >= 800
+#if LDC_LLVM_VER >= 800 && LDC_LLVM_VER < 1100
 // legacy option
 static cl::opt<bool>
     disableFPElim("disable-fp-elim", cl::ZeroOrMore, cl::ReallyHidden,
@@ -34,23 +36,43 @@ static cl::opt<bool>
 // in the opts namespace, including some additional helper functions.
 namespace opts {
 
-std::string getArchStr() { return ::MArch; }
-
-Optional<Reloc::Model> getRelocModel() { return ::getRelocModel(); }
-
-#if LDC_LLVM_VER >= 600
-Optional<CodeModel::Model> getCodeModel() { return ::getCodeModel(); }
+std::string getArchStr() {
+#if LDC_LLVM_VER >= 1100
+  return codegen::getMArch();
 #else
-CodeModel::Model getCodeModel() { return ::CMModel; }
+  return ::MArch;
 #endif
+}
+
+Optional<Reloc::Model> getRelocModel() {
+#if LDC_LLVM_VER >= 1100
+  return codegen::getExplicitRelocModel();
+#else
+  return ::getRelocModel();
+#endif
+}
+
+Optional<CodeModel::Model> getCodeModel() {
+#if LDC_LLVM_VER >= 1100
+  return codegen::getExplicitCodeModel();
+#else
+  return ::getCodeModel();
+#endif
+}
 
 #if LDC_LLVM_VER >= 800
 llvm::Optional<llvm::FramePointer::FP> framePointerUsage() {
+#if LDC_LLVM_VER >= 1100
+  // Defaults to `FP::None`; no way to check if set explicitly by user except
+  // indirectly via setFunctionAttributes()...
+  return codegen::getFramePointerUsage();
+#else
   if (::FramePointerUsage.getNumOccurrences() > 0)
     return ::FramePointerUsage.getValue();
   if (disableFPElim.getNumOccurrences() > 0)
     return disableFPElim ? llvm::FramePointer::All : llvm::FramePointer::None;
   return llvm::None;
+#endif
 }
 #else
 cl::boolOrDefault disableFPElim() {
@@ -63,6 +85,10 @@ cl::boolOrDefault disableFPElim() {
 bool disableRedZone() { return ::DisableRedZone; }
 
 bool printTargetFeaturesHelp() {
+#if LDC_LLVM_VER >= 1100
+  const auto MCPU = codegen::getMCPU();
+  const auto MAttrs = codegen::getMAttrs();
+#endif
   if (MCPU == "help")
     return true;
   return std::any_of(MAttrs.begin(), MAttrs.end(),
@@ -70,15 +96,43 @@ bool printTargetFeaturesHelp() {
 }
 
 TargetOptions InitTargetOptionsFromCodeGenFlags() {
+#if LDC_LLVM_VER >= 1100
+  return codegen::InitTargetOptionsFromCodeGenFlags();
+#else
   return ::InitTargetOptionsFromCodeGenFlags();
+#endif
 }
 
-std::string getCPUStr() { return ::getCPUStr(); }
-std::string getFeaturesStr() { return ::getFeaturesStr(); }
+std::string getCPUStr() {
+#if LDC_LLVM_VER >= 1100
+  return codegen::getCPUStr();
+#else
+  return ::getCPUStr();
+#endif
+}
+
+std::string getFeaturesStr() {
+#if LDC_LLVM_VER >= 1100
+  return codegen::getFeaturesStr();
+#else
+  return ::getFeaturesStr();
+#endif
+}
+
+#if LDC_LLVM_VER >= 1000
+void setFunctionAttributes(StringRef cpu, StringRef features,
+                           Function &function) {
+#if LDC_LLVM_VER >= 1100
+  return codegen::setFunctionAttributes(cpu, features, function);
+#else
+  return ::setFunctionAttributes(cpu, features, function);
+#endif
+}
+#endif
 } // namespace opts
 
-#if LDC_WITH_LLD && LDC_LLVM_VER >= 500
-// LLD 5.0 uses the shared header too (for LTO) and exposes some wrappers in
+#if LDC_WITH_LLD
+// LLD uses the shared header too (for LTO) and exposes some wrappers in
 // the lld namespace. Define them here to prevent the LLD object from being
 // linked in with its conflicting command-line options.
 namespace lld {
@@ -87,35 +141,35 @@ TargetOptions initTargetOptionsFromCodeGenFlags() {
 #else
 TargetOptions InitTargetOptionsFromCodeGenFlags() {
 #endif
-  return ::InitTargetOptionsFromCodeGenFlags();
+  return ::opts::InitTargetOptionsFromCodeGenFlags();
 }
 
 #if LDC_LLVM_VER >= 1000
-Optional<Reloc::Model> getRelocModelFromCMModel() { return ::getRelocModel(); }
+Optional<Reloc::Model> getRelocModelFromCMModel() {
+  return ::opts::getRelocModel();
+}
 #endif
 
 #if LDC_LLVM_VER >= 900
 Optional<CodeModel::Model> getCodeModelFromCMModel() {
-  return ::getCodeModel();
-}
-#elif LDC_LLVM_VER >= 600
-Optional<CodeModel::Model> GetCodeModelFromCMModel() {
-  return ::getCodeModel();
-}
 #else
-CodeModel::Model GetCodeModelFromCMModel() { return ::CMModel; }
+Optional<CodeModel::Model> GetCodeModelFromCMModel() {
 #endif
+  return ::opts::getCodeModel();
+}
 
 #if LDC_LLVM_VER >= 900
-std::string getCPUStr() { return ::getCPUStr(); }
+std::string getCPUStr() { return ::opts::getCPUStr(); }
 #elif LDC_LLVM_VER >= 700
-std::string GetCPUStr() { return ::getCPUStr(); }
+std::string GetCPUStr() { return ::opts::getCPUStr(); }
 #endif
 
-#if LDC_LLVM_VER >= 900
+#if LDC_LLVM_VER >= 1100
+std::vector<std::string> getMAttrs() { return codegen::getMAttrs(); }
+#elif LDC_LLVM_VER >= 900
 std::vector<std::string> getMAttrs() { return ::MAttrs; }
 #elif LDC_LLVM_VER >= 800
 std::vector<std::string> GetMAttrs() { return ::MAttrs; }
 #endif
 } // namespace lld
-#endif // LDC_WITH_LLD && LDC_LLVM_VER >= 500
+#endif // LDC_WITH_LLD
