@@ -1,6 +1,6 @@
 //===-- driver/cl_options-llvm.h - LLVM command line options ----*- C++ -*-===//
 //
-//                         LDC  the LLVM D compiler
+//                         LDC – the LLVM D compiler
 //
 // This file is distributed under the BSD-style LDC license. See the LICENSE
 // file for details.
@@ -14,15 +14,15 @@
 #include "llvm/Support/CodeGen.h"
 #include "llvm/Target/TargetOptions.h"
 
+namespace llvm {
+class Function;
+}
+
 namespace opts {
 
 std::string getArchStr();
 llvm::Optional<llvm::Reloc::Model> getRelocModel();
-#if LDC_LLVM_VER >= 600
 llvm::Optional<llvm::CodeModel::Model> getCodeModel();
-#else
-llvm::CodeModel::Model getCodeModel();
-#endif
 #if LDC_LLVM_VER >= 800
 llvm::Optional<llvm::FramePointer::FP> framePointerUsage();
 #else
@@ -35,4 +35,8 @@ bool printTargetFeaturesHelp();
 llvm::TargetOptions InitTargetOptionsFromCodeGenFlags();
 std::string getCPUStr();
 std::string getFeaturesStr();
+#if LDC_LLVM_VER >= 1000
+void setFunctionAttributes(llvm::StringRef cpu, llvm::StringRef features,
+                           llvm::Function &function);
+#endif
 }
