@@ -23,33 +23,6 @@ llvm::SmallVector<const char *, 32> allArguments;
 
 cl::OptionCategory linkingCategory("Linking options");
 
-/* Option parser that defaults to zero when no explicit number is given.
- * i.e.:  -cov    --> value = 0
- *        -cov=9  --> value = 9
- *        -cov=101 --> error, value must be in range [0..100]
- */
-struct CoverageParser : public cl::parser<unsigned char> {
-  explicit CoverageParser(cl::Option &O) : cl::parser<unsigned char>(O) {}
-
-  bool parse(cl::Option &O, llvm::StringRef /*ArgName*/, llvm::StringRef Arg,
-             unsigned char &Val) {
-    if (Arg == "") {
-      Val = 0;
-      return false;
-    }
-
-    if (Arg.getAsInteger(0, Val)) {
-      return O.error("'" + Arg +
-                     "' value invalid for required coverage percentage");
-    }
-
-    if (Val > 100) {
-      return O.error("Required coverage percentage must be <= 100");
-    }
-    return false;
-  }
-};
-
 // Positional options first, in order:
 cl::list<std::string> fileList(cl::Positional, cl::desc("files"));
 
@@ -65,7 +38,7 @@ cl::opt<bool> invokedByLDMD("ldmd", cl::desc("Invoked by LDMD?"),
 static cl::opt<Diagnostic, true> useDeprecated(
     cl::desc("Allow deprecated language features and symbols:"), cl::ZeroOrMore,
     cl::location(global.params.useDeprecated), cl::init(DIAGNOSTICinform),
-    clEnumValues(
+    cl::values(
         clEnumValN(DIAGNOSTICoff, "d",
                    "Silently allow deprecated features and symbols"),
         clEnumValN(DIAGNOSTICinform, "dw",
@@ -90,9 +63,9 @@ static cl::opt<bool, true>
 cl::opt<unsigned char> defaultToHiddenVisibility(
     "fvisibility", cl::ZeroOrMore,
     cl::desc("Default visibility of symbols (not relevant for Windows)"),
-    clEnumValues(clEnumValN(0, "default", "Export all symbols"),
-                 clEnumValN(1, "hidden",
-                            "Only export symbols marked with 'export'")));
+    cl::values(clEnumValN(0, "default", "Export all symbols"),
+               clEnumValN(1, "hidden",
+                          "Only export symbols marked with 'export'")));
 
 static cl::opt<bool, true> verbose("v", cl::desc("Verbose"), cl::ZeroOrMore,
                                    cl::location(global.params.verbose));
@@ -106,6 +79,41 @@ static cl::opt<bool, true>
     vgc("vgc", cl::desc("List all gc allocations including hidden ones"),
         cl::ZeroOrMore, cl::location(global.params.vgc));
 
+// Dummy data type for custom parsers where the help output shouldn't display
+// any value. cl::parser<bool> is final for LLVM < 9...
+#if LDC_LLVM_VER >= 900
+using DummyDataType = bool;
+#else
+enum class DummyDataType { dummy };
+#endif
+
+// `-vtemplates[=list-instances]` parser.
+struct VTemplatesParser : public cl::parser<DummyDataType> {
+  explicit VTemplatesParser(cl::Option &O) : cl::parser<DummyDataType>(O) {}
+
+  bool parse(cl::Option &O, llvm::StringRef /*ArgName*/, llvm::StringRef Arg,
+             DummyDataType & /*Val*/) {
+    global.params.vtemplates = true;
+
+    if (Arg.empty()) {
+      return false;
+    }
+
+    if (Arg == "list-instances") {
+      global.params.vtemplatesListInstances = true;
+      return false;
+    }
+
+    return O.error("unsupported value '" + Arg + "'");
+  }
+};
+
+static cl::opt<DummyDataType, false, VTemplatesParser> vtemplates(
+    "vtemplates", cl::ZeroOrMore, cl::ValueOptional,
+    cl::desc("List statistics on template instantiations\n"
+             "Use -vtemplates=list-instances to additionally show all "
+             "instantiation contexts for each template"));
+
 static cl::opt<bool, true> verbose_cg("v-cg", cl::desc("Verbose codegen"),
                                       cl::ZeroOrMore,
                                       cl::location(global.params.verbose_cg));
@@ -134,7 +142,7 @@ static cl::opt<MessageStyle, true> verrorStyle(
     "verror-style", cl::ZeroOrMore, cl::location(global.params.messageStyle),
     cl::desc(
         "Set the style for file/line number annotations on compiler messages"),
-    clEnumValues(
+    cl::values(
         clEnumValN(MESSAGESTYLEdigitalmars, "digitalmars",
                    "'file(line[,column]): message' (default)"),
         clEnumValN(MESSAGESTYLEgnu, "gnu",
@@ -144,7 +152,7 @@ static cl::opt<MessageStyle, true> verrorStyle(
 
 static cl::opt<Diagnostic, true> warnings(
     cl::desc("Warnings:"), cl::ZeroOrMore, cl::location(global.params.warnings),
-    clEnumValues(
+    cl::values(
         clEnumValN(DIAGNOSTICerror, "w",
                    "Enable warnings as errors (compilation will halt)"),
         clEnumValN(DIAGNOSTICinform, "wi",
@@ -159,7 +167,7 @@ static cl::opt<CppStdRevision, true> cplusplus(
     "extern-std", cl::ZeroOrMore,
     cl::desc("C++ standard for name mangling compatibility"),
     cl::location(global.params.cplusplus),
-    clEnumValues(
+    cl::values(
         clEnumValN(CppStdRevisionCpp98, "c++98",
                    "Sets `__traits(getTargetInfo, \"cppStd\")` to `199711`"),
         clEnumValN(CppStdRevisionCpp11, "c++11",
@@ -171,13 +179,17 @@ static cl::opt<CppStdRevision, true> cplusplus(
 
 static cl::opt<unsigned char, true> debugInfo(
     cl::desc("Generating debug information:"), cl::ZeroOrMore,
-    clEnumValues(
+    cl::values(
         clEnumValN(1, "g", "Add symbolic debug info"),
         clEnumValN(2, "gc",
                    "Add symbolic debug info, optimize for non D debuggers"),
         clEnumValN(3, "gline-tables-only", "Add line tables only")),
     cl::location(global.params.symdebug), cl::init(0));
 
+cl::opt<bool> emitDwarfDebugInfo(
+    "gdwarf", cl::ZeroOrMore,
+    cl::desc("Emit DWARF debuginfo (instead of CodeView) for MSVC targets"));
+
 cl::opt<bool> noAsm("noasm", cl::desc("Disallow use of inline assembler"),
                     cl::ZeroOrMore);
 
@@ -204,6 +216,9 @@ cl::opt<bool> output_bc("output-bc", cl::desc("Write LLVM bitcode"),
 
 cl::opt<bool> output_ll("output-ll", cl::desc("Write LLVM IR"), cl::ZeroOrMore);
 
+cl::opt<bool> output_mlir("output-mlir", cl::desc("Write MLIR"),
+    cl::ZeroOrMore);
+
 cl::opt<bool> output_s("output-s", cl::desc("Write native assembly"),
                        cl::ZeroOrMore);
 
@@ -258,9 +273,35 @@ cl::opt<bool>
                      cl::desc("Keep all function bodies in .di files"));
 
 // C++ header generation options
-static cl::opt<bool, true>
-    doCxxHdrGen("HC", cl::desc("Generate C++ 'header' file"), cl::ZeroOrMore,
-             cl::location(global.params.doCxxHdrGeneration));
+
+// `-HC[=silent|verbose]` parser. Required for defaulting to `silent`.
+struct HCParser : public cl::parser<CxxHeaderMode> {
+  explicit HCParser(cl::Option &O) : cl::parser<CxxHeaderMode>(O) {}
+
+  bool parse(cl::Option &O, llvm::StringRef /*ArgName*/, llvm::StringRef Arg,
+             CxxHeaderMode &Val) {
+    if (Arg.empty() || Arg == "silent") {
+      Val = CxxHeaderMode::silent;
+      return false;
+    }
+    if (Arg == "verbose") {
+      Val = CxxHeaderMode::verbose;
+      return false;
+    }
+
+    return O.error("unsupported value '" + Arg + "'");
+  }
+};
+
+static cl::opt<CxxHeaderMode, true, HCParser> doCxxHdrGen(
+    "HC", cl::ZeroOrMore, cl::desc("Generate C++ header file"),
+    cl::location(global.params.doCxxHdrGeneration), cl::ValueOptional,
+    cl::values(
+        clEnumValN(CxxHeaderMode::silent, "silent",
+                   "Only list extern(C[++]) declarations (default)"),
+        clEnumValN(
+            CxxHeaderMode::verbose, "verbose",
+            "Also add comments for ignored declarations (e.g. extern(D))")));
 
 cl::opt<std::string>
     cxxHdrDir("HCd", cl::ZeroOrMore, cl::Prefix,
@@ -321,17 +362,17 @@ cl::list<std::string> versions(
     cl::desc("Compile in version code >= <level> or identified by <idents>"));
 
 cl::list<std::string> transitions(
-    "transition", cl::CommaSeparated, cl::value_desc("id"),
-    cl::desc("Help with language change identified by <id>, use ? for list"));
+    "transition", cl::CommaSeparated, cl::value_desc("name"),
+    cl::desc("Help with language change identified by <name>, use ? for list"));
 
-cl::list<std::string> previews("preview", cl::CommaSeparated,
-                               cl::value_desc("id"),
-                               cl::desc("Enable an upcoming language change "
-                                        "identified by <id>, use ? for list"));
+cl::list<std::string>
+    previews("preview", cl::CommaSeparated, cl::value_desc("name"),
+             cl::desc("Enable an upcoming language change "
+                      "identified by <name>, use ? for list"));
 
 cl::list<std::string> reverts(
-    "revert", cl::CommaSeparated, cl::value_desc("id"),
-    cl::desc("Revert language change identified by <id>, use ? for list"));
+    "revert", cl::CommaSeparated, cl::value_desc("name"),
+    cl::desc("Revert language change identified by <name>, use ? for list"));
 
 cl::list<std::string>
     linkerSwitches("L", cl::desc("Pass <linkerflag> to the linker"),
@@ -339,8 +380,7 @@ cl::list<std::string>
                    cl::Prefix);
 
 cl::list<std::string>
-    ccSwitches("Xcc", cl::CommaSeparated,
-               cl::desc("Pass <ccflag> to GCC/Clang for linking"),
+    ccSwitches("Xcc", cl::desc("Pass <ccflag> to GCC/Clang for linking"),
                cl::value_desc("ccflag"), cl::cat(linkingCategory));
 
 cl::opt<std::string>
@@ -385,10 +425,10 @@ static cl::opt<CHECKENABLE, true, FlagParser<CHECKENABLE>>
 static cl::opt<CHECKENABLE, true> boundsCheck(
     "boundscheck", cl::ZeroOrMore, cl::desc("Array bounds check"),
     cl::location(global.params.useArrayBounds), cl::init(CHECKENABLEdefault),
-    clEnumValues(clEnumValN(CHECKENABLEoff, "off", "Disabled"),
-                 clEnumValN(CHECKENABLEsafeonly, "safeonly",
-                            "Enabled for @safe functions only"),
-                 clEnumValN(CHECKENABLEon, "on", "Enabled for all functions")));
+    cl::values(clEnumValN(CHECKENABLEoff, "off", "Disabled"),
+               clEnumValN(CHECKENABLEsafeonly, "safeonly",
+                          "Enabled for @safe functions only"),
+               clEnumValN(CHECKENABLEon, "on", "Enabled for all functions")));
 
 static cl::opt<CHECKENABLE, true, FlagParser<CHECKENABLE>> switchErrors(
     "switch-errors", cl::ZeroOrMore,
@@ -422,7 +462,7 @@ static cl::opt<CHECKACTION, true> checkAction(
     "checkaction", cl::ZeroOrMore, cl::location(global.params.checkAction),
     cl::desc("Action to take when an assert/boundscheck/final-switch fails"),
     cl::init(CHECKACTION_D),
-    clEnumValues(
+    cl::values(
         clEnumValN(CHECKACTION_D, "D",
                    "Usual D behavior of throwing an AssertError"),
         clEnumValN(CHECKACTION_C, "C",
@@ -457,16 +497,16 @@ cl::opt<bool> disableLinkerStripDead(
     cl::desc("Do not try to remove unused symbols during linking"),
     cl::cat(linkingCategory));
 
+cl::opt<bool> noPLT(
+    "fno-plt", cl::ZeroOrMore,
+    cl::desc("Do not use the PLT to make function calls"));
+
 // Math options
 bool fFastMath; // Storage for the dynamically created ffast-math option.
 llvm::FastMathFlags defaultFMF;
 void setDefaultMathOptions(llvm::TargetOptions &targetOptions) {
   if (fFastMath) {
-#if LDC_LLVM_VER >= 600
     defaultFMF.setFast();
-#else
-    defaultFMF.setUnsafeAlgebra();
-#endif
     targetOptions.UnsafeFPMath = true;
   }
 }
@@ -497,28 +537,58 @@ cl::opt<bool, true> betterC(
     "betterC", cl::ZeroOrMore, cl::location(global.params.betterC),
     cl::desc("Omit generating some runtime information and helper functions"));
 
-cl::opt<unsigned char, true, CoverageParser> coverageAnalysis(
-    "cov", cl::ZeroOrMore, cl::location(global.params.covPercent),
-    cl::desc("Compile-in code coverage analysis\n(use -cov=n for n% "
-             "minimum required coverage)"),
-    cl::ValueOptional, cl::init(127));
+// `-cov[=<n>|ctfe]` parser.
+struct CoverageParser : public cl::parser<DummyDataType> {
+  explicit CoverageParser(cl::Option &O) : cl::parser<DummyDataType>(O) {}
+
+  bool parse(cl::Option &O, llvm::StringRef /*ArgName*/, llvm::StringRef Arg,
+             DummyDataType & /*Val*/) {
+    global.params.cov = true;
+
+    if (Arg.empty()) {
+      return false;
+    }
+
+    if (Arg == "ctfe") {
+      global.params.ctfe_cov = true;
+      return false;
+    }
+
+    unsigned char percent = 0;
+    if (Arg.getAsInteger(0, percent)) {
+      return O.error("'" + Arg +
+                     "' value invalid for required coverage percentage");
+    }
+
+    if (percent > 100) {
+      return O.error("required coverage percentage must be <= 100");
+    }
+
+    global.params.covPercent = percent;
+    return false;
+  }
+};
+
+static cl::opt<DummyDataType, false, CoverageParser> coverageAnalysis(
+    "cov", cl::ZeroOrMore, cl::ValueOptional,
+    cl::desc("Compile-in code coverage analysis and .lst file generation\n"
+             "Use -cov=<n> for n% minimum required coverage\n"
+             "Use -cov=ctfe to include code executed during CTFE"));
 
 cl::opt<LTOKind> ltoMode(
     "flto", cl::ZeroOrMore, cl::desc("Set LTO mode, requires linker support"),
     cl::init(LTO_None),
-    clEnumValues(
+    cl::values(
         clEnumValN(LTO_Full, "full", "Merges all input into a single module"),
         clEnumValN(LTO_Thin, "thin",
                    "Parallel importing and codegen (faster than 'full')")));
 
-#if LDC_LLVM_VER >= 400
 cl::opt<std::string>
     saveOptimizationRecord("fsave-optimization-record",
                            cl::value_desc("filename"),
                            cl::desc("Generate a YAML optimization record file "
                                     "of optimizations performed by LLVM"),
                            cl::ValueOptional);
-#endif
 
 #if LDC_LLVM_SUPPORTED_TARGET_SPIRV || LDC_LLVM_SUPPORTED_TARGET_NVPTX
 cl::list<std::string>
@@ -592,7 +662,7 @@ void createClashingOptions() {
   new cl::opt<FloatABI::Type, true>(
       "float-abi", cl::desc("ABI/operations to use for floating-point types:"),
       cl::ZeroOrMore, cl::location(floatABI), cl::init(FloatABI::Default),
-      clEnumValues(
+      cl::values(
           clEnumValN(FloatABI::Default, "default",
                      "Target default floating-point ABI"),
           clEnumValN(FloatABI::Soft, "soft",
@@ -610,17 +680,24 @@ void hideLLVMOptions() {
   static const char *const hiddenOptions[] = {
       "aarch64-neon-syntax", "addrsig", "arm-add-build-attributes",
       "arm-implicit-it", "asm-instrumentation", "asm-show-inst",
-      "atomic-counter-update-promoted", "bounds-checking-single-trap",
+      "atomic-counter-update-promoted", "atomic-first-counter",
+      "basicblock-sections", "bounds-checking-single-trap",
+      "cfg-hide-deoptimize-paths", "cfg-hide-unreachable-paths",
       "code-model", "cost-kind", "cppfname", "cppfor", "cppgen",
       "cvp-dont-add-nowrap-flags",
       "cvp-dont-process-adds", "debug-counter", "debug-entry-values",
-      "debugger-tune", "denormal-fp-math", "disable-debug-info-verifier",
+      "debugger-tune", "debugify-level", "debugify-quiet",
+      "denormal-fp-math", "denormal-fp-math-f32", "disable-debug-info-verifier",
       "disable-objc-arc-checkforcfghazards", "disable-spill-fusing",
-      "do-counter-promotion", "emscripten-cxx-exceptions-whitelist",
+      "do-counter-promotion", "dwarf64", "emit-call-site-info",
+      "emscripten-cxx-exceptions-allowed",
+      "emscripten-cxx-exceptions-whitelist",
       "emulated-tls", "enable-correct-eh-support",
       "enable-cse-in-irtranslator", "enable-cse-in-legalizer",
       "enable-emscripten-cxx-exceptions", "enable-emscripten-sjlj",
-      "enable-fp-mad", "enable-gvn-memdep", "enable-implicit-null-checks",
+      "enable-fp-mad", "enable-gvn-hoist", "enable-gvn-memdep",
+      "enable-gvn-sink", "enable-implicit-null-checks",
+      "enable-load-in-loop-pre",
       "enable-load-pre", "enable-loop-simplifycfg-term-folding",
       "enable-misched", "enable-name-compression", "enable-no-infs-fp-math",
       "enable-no-nans-fp-math", "enable-no-signed-zeros-fp-math",
@@ -630,23 +707,27 @@ void hideLLVMOptions() {
       "exhaustive-register-search", "expensive-combines",
       "fatal-assembler-warnings", "filter-print-funcs",
       "force-dwarf-frame-section", "gpsize", "hash-based-counter-split",
+      "hot-cold-split",
       "imp-null-check-page-size", "imp-null-max-insts-to-consider",
       "import-all-index", "incremental-linker-compatible",
       "instcombine-code-sinking", "instcombine-guard-widening-window",
       "instcombine-max-iterations", "instcombine-max-num-phis",
-      "instcombine-maxarray-size", "instrprof-atomic-counter-update-all",
-      "internalize-public-api-file",
+      "instcombine-maxarray-size", "instcombine-negator-enabled",
+      "instcombine-negator-max-depth",
+      "instrprof-atomic-counter-update-all", "internalize-public-api-file",
       "internalize-public-api-list", "iterative-counter-promotion",
       "join-liveintervals", "jump-table-type", "limit-float-precision",
-      "lto-embed-bitcode", "matrix-propagate-shape",
+      "lto-embed-bitcode", "matrix-default-layout", "matrix-propagate-shape",
       "max-counter-promotions", "max-counter-promotions-per-loop",
       "mc-relax-all", "mc-x86-disable-arith-relaxation", "meabi",
       "memop-size-large", "memop-size-range", "merror-missing-parenthesis",
       "merror-noncontigious-register", "mfuture-regs", "mips-compact-branches",
-      "mips16-constant-islands", "mips16-hard-float", "mlsm", "mno-compound",
+      "mips16-constant-islands", "mips16-hard-float", "mir-strip-debugify-only",
+      "mlsm", "mno-compound",
       "mno-fixup", "mno-ldc1-sdc1", "mno-pairing", "mwarn-missing-parenthesis",
       "mwarn-noncontigious-register", "mwarn-sign-mismatch",
-      "no-discriminators", "nozero-initialized-in-bss", "nvptx-sched4reg",
+      "no-discriminators", "no-xray-index",
+      "nozero-initialized-in-bss", "nvptx-sched4reg",
       "objc-arc-annotation-target-identifier", "pie-copy-relocations",
       "poison-checking-function-local",
       "polly-dump-after", "polly-dump-after-file", "polly-dump-before",
@@ -656,7 +737,7 @@ void hideLLVMOptions() {
       "profile-file", "profile-info-file", "profile-verifier-noassert",
       "r600-ir-structurize", "rdf-dump", "rdf-limit", "recip", "regalloc",
       "relax-elf-relocations", "remarks-section", "rewrite-map-file", "rng-seed",
-      "safepoint-ir-verifier-print-only",
+      "runtime-counter-relocation", "safepoint-ir-verifier-print-only",
       "sample-profile-check-record-coverage",
       "sample-profile-check-sample-coverage",
       "sample-profile-inline-hot-threshold",
@@ -664,14 +745,15 @@ void hideLLVMOptions() {
       "speculative-counter-promotion-max-exiting",
       "speculative-counter-promotion-to-loop", "spiller", "spirv-debug",
       "spirv-erase-cl-md", "spirv-lower-const-expr", "spirv-mem2reg",
-      "spirv-no-deref-attr", "spirv-text", "spvbool-validate",
+      "spirv-no-deref-attr", "spirv-text",
+      "spv-lower-saddwithoverflow-validate", "spvbool-validate",
       "spvmemmove-validate",
       "stack-alignment", "stack-size-section", "stack-symbol-ordering",
       "stackmap-version", "static-func-full-module-prefix",
       "static-func-strip-dirname-prefix", "stats", "stats-json", "strip-debug",
-      "struct-path-tbaa", "summary-file", "tailcallopt", "thread-model",
-      "time-passes", "time-trace-granularity", "tls-size",
-      "unfold-element-atomic-memcpy-max-elements",
+      "struct-path-tbaa", "summary-file", "tail-predication", "tailcallopt",
+      "thread-model", "time-passes", "time-trace-granularity", "tls-size",
+      "unfold-element-atomic-memcpy-max-elements", "unique-bb-section-names",
       "unique-section-names", "unit-at-a-time", "use-ctors",
       "verify-debug-info", "verify-dom-info", "verify-loop-info",
       "verify-loop-lcssa", "verify-machine-dom-info", "verify-regalloc",
@@ -679,6 +761,7 @@ void hideLLVMOptions() {
       "vp-counters-per-site", "vp-static-alloc",
       "x86-align-branch", "x86-align-branch-boundary",
       "x86-branches-within-32B-boundaries", "x86-early-ifcvt",
+      "x86-pad-max-prefix-size",
       "x86-recip-refinement-steps", "x86-use-vzeroupper",
 
       // We enable -fdata-sections/-ffunction-sections by default where it makes
@@ -687,8 +770,7 @@ void hideLLVMOptions() {
       // We need our own switch as these two are defined by LLVM and linked to
       // static TargetMachine members, but the default we want to use depends
       // on the target triple (and thus we do not know it until after the
-      // command
-      // line has been parsed).
+      // command line has been parsed).
       "fdata-sections", "ffunction-sections", "data-sections",
       "function-sections"};
 
