@@ -23,7 +23,9 @@
 #include "gen/logger.h"
 #include "gen/modules.h"
 #include "gen/runtime.h"
-#if LDC_LLVM_VER >= 900
+#if LDC_LLVM_VER >= 1100
+#include "llvm/IR/LLVMRemarkStreamer.h"
+#elif LDC_LLVM_VER >= 900
 #include "llvm/IR/RemarkStreamer.h"
 #endif
 #include "llvm/Support/FileSystem.h"
@@ -31,12 +33,6 @@
 #include "llvm/Support/ToolOutputFile.h"
 #include "llvm/Support/YAMLTraits.h"
 
-#if LDC_LLVM_VER < 600
-namespace llvm {
-using ToolOutputFile = tool_output_file;
-}
-#endif
-
 namespace {
 
 std::unique_ptr<llvm::ToolOutputFile>
@@ -44,7 +40,6 @@ createAndSetDiagnosticsOutputFile(IRState &irs, llvm::LLVMContext &ctx,
                                   llvm::StringRef filename) {
   std::unique_ptr<llvm::ToolOutputFile> diagnosticsOutputFile;
 
-#if LDC_LLVM_VER >= 400
   // Set LLVM Diagnostics outputfile if requested
   if (opts::saveOptimizationRecord.getNumOccurrences() > 0) {
     llvm::SmallString<128> diagnosticsFilename;
@@ -59,8 +54,13 @@ createAndSetDiagnosticsOutputFile(IRState &irs, llvm::LLVMContext &ctx,
     const bool withHotness = opts::isUsingPGOProfile();
 
 #if LDC_LLVM_VER >= 900
-    auto remarksFileOrError = llvm::setupOptimizationRemarks(
-        ctx, diagnosticsFilename, "", "", withHotness);
+    auto remarksFileOrError =
+#if LDC_LLVM_VER >= 1100
+        llvm::setupLLVMOptimizationRemarks(
+#else
+        llvm::setupOptimizationRemarks(
+#endif
+            ctx, diagnosticsFilename, "", "", withHotness);
     if (llvm::Error e = remarksFileOrError.takeError()) {
       irs.dmodule->error("Could not create file %s: %s",
                          diagnosticsFilename.c_str(),
@@ -82,63 +82,14 @@ createAndSetDiagnosticsOutputFile(IRState &irs, llvm::LLVMContext &ctx,
         llvm::make_unique<llvm::yaml::Output>(diagnosticsOutputFile->os()));
 
     if (withHotness) {
-#if LDC_LLVM_VER >= 500
       ctx.setDiagnosticsHotnessRequested(true);
-#else
-      ctx.setDiagnosticHotnessRequested(true);
-#endif
     }
 #endif // LDC_LLVM_VER < 900
   }
-#endif
 
   return diagnosticsOutputFile;
 }
 
-#if LDC_LLVM_VER < 500
-/// Add the Linker Options module flag.
-/// If the flag is already present, merge it with the new data.
-void emitLinkerOptions(IRState &irs) {
-  llvm::Module &M = irs.module;
-  llvm::LLVMContext &ctx = irs.context();
-  if (!M.getModuleFlag("Linker Options")) {
-    M.addModuleFlag(llvm::Module::AppendUnique, "Linker Options",
-                    llvm::MDNode::get(ctx, irs.linkerOptions));
-  } else {
-    // Merge the Linker Options with the pre-existing one
-    // (this can happen when passing a .bc file on the commandline)
-
-    auto *moduleFlags = M.getModuleFlagsMetadata();
-    for (unsigned i = 0, e = moduleFlags->getNumOperands(); i < e; ++i) {
-      auto *flag = moduleFlags->getOperand(i);
-      if (flag->getNumOperands() < 3)
-        continue;
-      auto optionsMDString =
-          llvm::dyn_cast_or_null<llvm::MDString>(flag->getOperand(1));
-      if (!optionsMDString || optionsMDString->getString() != "Linker Options")
-        continue;
-
-      // If we reach here, we found the Linker Options flag.
-
-      // Add the old Linker Options to our linkerOptions list.
-      auto *oldLinkerOptions = llvm::cast<llvm::MDNode>(flag->getOperand(2));
-      for (const auto &Option : oldLinkerOptions->operands()) {
-        irs.linkerOptions.push_back(Option);
-      }
-
-      // Replace Linker Options with a newly created list.
-      llvm::Metadata *Ops[3] = {
-          llvm::ConstantAsMetadata::get(llvm::ConstantInt::get(
-              llvm::Type::getInt32Ty(ctx), llvm::Module::AppendUnique)),
-          llvm::MDString::get(ctx, "Linker Options"),
-          llvm::MDNode::get(ctx, irs.linkerOptions)};
-      moduleFlags->setOperand(i, llvm::MDNode::get(ctx, Ops));
-
-      break;
-    }
-  }
-}
-#else
 void addLinkerMetadata(llvm::Module &M, const char *name,
                        llvm::ArrayRef<llvm::MDNode *> newOperands) {
   if (newOperands.empty())
@@ -172,7 +123,6 @@ void emitLinkerOptions(IRState &irs) {
   addLinkerMetadata(M, "llvm.linker.options", irs.linkerOptions);
   addLinkerMetadata(M, "llvm.dependent-libraries", irs.linkerDependentLibs);
 }
-#endif
 
 void emitLLVMUsedArray(IRState &irs) {
   if (irs.usedArray.empty()) {
@@ -219,8 +169,16 @@ void inlineAsmDiagnosticHandler(const llvm::SMDiagnostic &d, void *context,
 } // anonymous namespace
 
 namespace ldc {
-CodeGenerator::CodeGenerator(llvm::LLVMContext &context, bool singleObj)
-    : context_(context), moduleCount_(0), singleObj_(singleObj), ir_(nullptr) {
+CodeGenerator::CodeGenerator(llvm::LLVMContext &context,
+#if LDC_MLIR_ENABLED
+                             mlir::MLIRContext &mlirContext,
+#endif
+                             bool singleObj)
+    : context_(context),
+#if LDC_MLIR_ENABLED
+      mlirContext_(mlirContext),
+#endif
+      moduleCount_(0), singleObj_(singleObj), ir_(nullptr) {
   // Set the context to discard value names when not generating textual IR.
   if (!global.params.output_ll) {
     context_.setDiscardValueNames(true);
@@ -274,7 +232,6 @@ void CodeGenerator::finishLLModule(Module *m) {
   if (moduleCount_ == 1) {
     insertBitcodeFiles(ir_->module, ir_->context(), global.params.bitcodeFiles);
   }
-
   writeAndFreeLLModule(m->objfile.toChars());
 }
 
@@ -341,4 +298,58 @@ void CodeGenerator::emit(Module *m) {
     Logger::disable();
   }
 }
+
+#if LDC_MLIR_ENABLED
+void CodeGenerator::emitMLIR(Module *m) {
+  bool const loggerWasEnabled = Logger::enabled();
+  if (m->llvmForceLogging && !loggerWasEnabled) {
+    Logger::enable();
+  }
+
+  IF_LOG Logger::println("CodeGenerator::emitMLIR(%s)", m->toPrettyChars());
+  LOG_SCOPE;
+
+  if (global.params.verbose_cg) {
+    printf("codegen: %s (%s)\n", m->toPrettyChars(), m->srcfile.toChars());
+  }
+
+  if (global.errors) {
+    Logger::println("Aborting because of errors");
+    fatal();
+  }
+
+  mlir::OwningModuleRef module;
+  /*module = mlirGen(mlirContext, m, irs);
+  if(!module){
+    IF_LOG Logger::println("Error generating MLIR:'%s'", llpath.c_str());
+    fatal();
+  }*/
+
+  writeMLIRModule(&module, m->objfile.toChars());
+
+  if (m->llvmForceLogging && !loggerWasEnabled) {
+    Logger::disable();
+  }
+}
+
+void CodeGenerator::writeMLIRModule(mlir::OwningModuleRef *module,
+                                    const char *filename) {
+  // Write MLIR
+  if (global.params.output_mlir) {
+    const auto llpath = replaceExtensionWith(global.mlir_ext, filename);
+    Logger::println("Writting MLIR to %s\n", llpath.c_str());
+    std::error_code errinfo;
+    llvm::raw_fd_ostream aos(llpath, errinfo, llvm::sys::fs::F_None);
+
+    if (aos.has_error()) {
+      error(Loc(), "Cannot write MLIR file '%s': %s", llpath.c_str(),
+            errinfo.message().c_str());
+      fatal();
+    }
+
+    // module->print(aos);
+  }
+}
+
+#endif
 }
