@@ -8,6 +8,7 @@
 //===----------------------------------------------------------------------===//
 
 #include "dmd/errors.h"
+#include "driver/args.h"
 #include "driver/cl_options.h"
 #include "driver/cl_options_instrumentation.h"
 #include "driver/cl_options_sanitizers.h"
@@ -20,32 +21,28 @@
 #include "llvm/Support/Path.h"
 
 #if LDC_WITH_LLD
-#if LDC_LLVM_VER >= 600
 #include "lld/Common/Driver.h"
-#else
-#include "lld/Driver/Driver.h"
-#endif
 #endif
 
 //////////////////////////////////////////////////////////////////////////////
 
 namespace {
 
-void addMscrtLibs(std::vector<std::string> &args) {
-  const auto mscrtlibName = getMscrtLibName();
+void addMscrtLibs(bool useInternalToolchain, std::vector<std::string> &args) {
+  const auto mscrtlibName = getMscrtLibName(&useInternalToolchain);
 
   args.push_back(("/DEFAULTLIB:" + mscrtlibName).str());
 
   // We need the vcruntime lib for druntime's exception handling (ldc.eh_msvc).
   // Pick one of the 4 variants matching the selected main UCRT lib.
 
-#if LDC_LLVM_VER >= 400
-  if (mscrtlibName.contains_lower("vcruntime")) {
+  if (useInternalToolchain) {
+    assert(mscrtlibName.contains_lower("vcruntime"));
     return;
   }
-#endif
 
-  const bool isStatic = mscrtlibName.startswith_lower("libcmt");
+  const bool isStatic = mscrtlibName.contains_lower("libcmt");
+
   const bool isDebug =
       mscrtlibName.endswith_lower("d") || mscrtlibName.endswith_lower("d.lib");
 
@@ -70,6 +67,10 @@ void addSanitizerLibs(std::vector<std::string> &args) {
   if (opts::isSanitizerEnabled(opts::AddressSanitizer)) {
     args.push_back("ldc_rt.asan.lib");
   }
+  if (opts::isSanitizerEnabled(opts::FuzzSanitizer)) {
+    args.push_back("ldc_rt.fuzzer.lib");
+    args.push_back("/SUBSYSTEM:CONSOLE"); // pull main() from fuzzer lib
+  }
 
   // TODO: remaining sanitizers
 }
@@ -85,12 +86,20 @@ int linkObjToBinaryMSVC(llvm::StringRef outputPath,
     fatal();
   }
 
-  const bool useInternalToolchain = useInternalToolchainForMSVC();
-
 #ifdef _WIN32
   windows::MsvcEnvironmentScope msvcEnv;
-  if (!useInternalToolchain)
-    msvcEnv.setup();
+
+  const bool forceMSVC = env::has(L"LDC_VSDIR_FORCE");
+  const bool useInternalToolchain =
+      (!forceMSVC && getExplicitMscrtLibName().contains_lower("vcruntime")) ||
+      !msvcEnv.setup();
+
+  if (forceMSVC && useInternalToolchain) {
+    warning(Loc(), "no Visual C++ installation found for linking, falling back "
+                   "to MinGW-based libraries");
+  }
+#else
+  const bool useInternalToolchain = true;
 #endif
 
   // build arguments
@@ -121,7 +130,7 @@ int linkObjToBinaryMSVC(llvm::StringRef outputPath,
   }
 
   // add C runtime libs
-  addMscrtLibs(args);
+  addMscrtLibs(useInternalToolchain, args);
 
   // specify creation of DLL
   if (global.params.dll) {
@@ -201,23 +210,24 @@ int linkObjToBinaryMSVC(llvm::StringRef outputPath,
     addSwitch(ls);
   }
 
-  // default platform libs
-  // TODO check which libaries are necessary
-  args.push_back("kernel32.lib");
-  args.push_back("user32.lib");
-  args.push_back("gdi32.lib");
-  args.push_back("winspool.lib");
-  args.push_back("shell32.lib"); // required for dmain2.d
-  args.push_back("ole32.lib");
-  args.push_back("oleaut32.lib");
-  args.push_back("uuid.lib");
-  args.push_back("comdlg32.lib");
-  args.push_back("advapi32.lib");
-
-  // these get pulled in by druntime (rt/msvc.c); include explicitly for
-  // -betterC convenience (issue #3035)
-  args.push_back("oldnames.lib");
-  args.push_back("legacy_stdio_definitions.lib");
+  auto explicitPlatformLibs = getExplicitPlatformLibs();
+  if (explicitPlatformLibs.hasValue()) {
+    for (auto &lib : explicitPlatformLibs.getValue()) {
+      args.push_back(llvm::sys::path::has_extension(lib) ? std::move(lib)
+                                                         : lib + ".lib");
+    }
+  } else {
+    // default platform libs
+    // TODO check which libaries are necessary
+    args.insert(args.end(),
+                {"kernel32.lib", "user32.lib", "gdi32.lib", "winspool.lib",
+                 "shell32.lib", // required for dmain2.d
+                 "ole32.lib", "oleaut32.lib", "uuid.lib", "comdlg32.lib",
+                 "advapi32.lib",
+                 // these get pulled in by druntime (rt/msvc.c); include
+                 // explicitly for -betterC convenience (issue #3035)
+                 "oldnames.lib", "legacy_stdio_definitions.lib"});
+  }
 
   Logger::println("Linking with: ");
   Stream logstr = Logger::cout();
@@ -233,14 +243,11 @@ int linkObjToBinaryMSVC(llvm::StringRef outputPath,
       (useInternalToolchain && opts::linker.empty() && !opts::isUsingLTO())) {
     const auto fullArgs = getFullArgs("lld-link", args, global.params.verbose);
 
-    const bool success = lld::coff::link(fullArgs
-#if LDC_LLVM_VER >= 600
-                                         ,
+    const bool success = lld::coff::link(fullArgs,
                                          /*CanExitEarly=*/false
 #if LDC_LLVM_VER >= 1000
                                          ,
                                          llvm::outs(), llvm::errs()
-#endif
 #endif
     );
 
