@@ -20,12 +20,8 @@
 #include "llvm/IR/AssemblyAnnotationWriter.h"
 #include "llvm/IR/Verifier.h"
 #include "llvm/Analysis/ModuleSummaryAnalysis.h"
-#if LDC_LLVM_VER >= 400
 #include "llvm/Analysis/ProfileSummaryInfo.h"
 #include "llvm/Bitcode/BitcodeWriter.h"
-#else
-#include "llvm/Bitcode/ReaderWriter.h"
-#endif
 #include "llvm/IR/LegacyPassManager.h"
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/FileSystem.h"
@@ -34,11 +30,7 @@
 #include "llvm/Support/Path.h"
 #include "llvm/Target/TargetMachine.h"
 #include "llvm/Analysis/TargetTransformInfo.h"
-#if LDC_LLVM_VER >= 600
 #include "llvm/CodeGen/TargetSubtargetInfo.h"
-#else
-#include "llvm/Target/TargetSubtargetInfo.h"
-#endif
 #include "llvm/Transforms/Utils/Cloning.h"
 #include "llvm/IR/Module.h"
 #ifdef LDC_LLVM_SUPPORTED_TARGET_SPIRV
@@ -171,13 +163,8 @@ class AssemblyAnnotator : public AssemblyAnnotationWriter {
   static llvm::StringRef GetDisplayName(const Function *F) {
     llvm::DebugInfoFinder Finder;
     Finder.processModule(*F->getParent());
-    if (DISubprogram *N = FindSubprogram(F, Finder))
-    {
-#if LDC_LLVM_VER >= 500
+    if (DISubprogram *N = FindSubprogram(F, Finder)) {
       return N->getName();
-#else
-      return N->getDisplayName();
-#endif
     }
     return "";
   }
@@ -296,21 +283,27 @@ bool shouldAssembleExternally() {
 bool shouldOutputObjectFile() {
   return global.params.output_o && !shouldAssembleExternally();
 }
-
-bool shouldDoLTO(llvm::Module *m) {
-#if LDC_LLVM_VER == 309
-  // LLVM 3.9 bug: can't do ThinLTO with modules that have module-scope inline
-  // assembly blocks (duplicate definitions upon importing from such a module).
-  // https://llvm.org/bugs/show_bug.cgi?id=30610
-  if (opts::isUsingThinLTO() && !m->getModuleInlineAsm().empty())
-    return false;
-#endif
-  return opts::isUsingLTO();
-}
 } // end of anonymous namespace
 
+std::string replaceExtensionWith(const DArray<const char> &ext,
+                                 const char *filename) {
+  const auto outputFlags = {global.params.output_o, global.params.output_bc,
+                            global.params.output_ll, global.params.output_s,
+                            global.params.output_mlir};
+  const auto numOutputFiles =
+      std::count_if(outputFlags.begin(), outputFlags.end(),
+                    [](OUTPUTFLAG flag) { return flag != 0; });
+
+  if (numOutputFiles == 1)
+    return filename;
+  llvm::SmallString<128> buffer(filename);
+  llvm::sys::path::replace_extension(buffer,
+                                     llvm::StringRef(ext.ptr, ext.length));
+  return {buffer.data(), buffer.size()};
+}
+
 void writeModule(llvm::Module *m, const char *filename) {
-  const bool doLTO = shouldDoLTO(m);
+  const bool doLTO = opts::isUsingLTO();
   const bool outputObj = shouldOutputObjectFile();
   const bool assembleExternally = shouldAssembleExternally();
 
@@ -349,29 +342,13 @@ void writeModule(llvm::Module *m, const char *filename) {
     }
   }
 
-  const auto outputFlags = {global.params.output_o, global.params.output_bc,
-                            global.params.output_ll, global.params.output_s};
-  const auto numOutputFiles =
-      std::count_if(outputFlags.begin(), outputFlags.end(),
-                    [](OUTPUTFLAG flag) { return flag != 0; });
-
-  const auto replaceExtensionWith =
-      [=](const DArray<const char> &ext) -> std::string {
-    if (numOutputFiles == 1)
-      return filename;
-    llvm::SmallString<128> buffer(filename);
-    llvm::sys::path::replace_extension(buffer,
-                                       llvm::StringRef(ext.ptr, ext.length));
-    return {buffer.data(), buffer.size()};
-  };
-
   // write LLVM bitcode
   const bool emitBitcodeAsObjectFile =
       doLTO && outputObj && !global.params.output_bc;
   if (global.params.output_bc || emitBitcodeAsObjectFile) {
     std::string bcpath = emitBitcodeAsObjectFile
                              ? filename
-                             : replaceExtensionWith(global.bc_ext);
+                             : replaceExtensionWith(global.bc_ext, filename);
     Logger::println("Writing LLVM bitcode to: %s\n", bcpath.c_str());
     std::error_code errinfo;
     llvm::raw_fd_ostream bos(bcpath.c_str(), errinfo, llvm::sys::fs::F_None);
@@ -389,20 +366,13 @@ void writeModule(llvm::Module *m, const char *filename) {
 
     if (opts::isUsingThinLTO()) {
       Logger::println("Creating module summary for ThinLTO");
-#if LDC_LLVM_VER == 309
-      // When the function freq info callback is set to nullptr, LLVM will
-      // calculate it automatically for us.
-      llvm::ModuleSummaryIndexBuilder indexBuilder(
-          m, /* function freq callback */ nullptr);
-      auto &moduleSummaryIndex = indexBuilder.getIndex();
-#else
+
       llvm::ProfileSummaryInfo PSI(*m);
 
       // When the function freq info callback is set to nullptr, LLVM will
       // calculate it automatically for us.
       auto moduleSummaryIndex = buildModuleSummaryIndex(
           *m, /* function freq callback */ nullptr, &PSI);
-#endif
 
       llvm::WriteBitcodeToFile(M, bos, true, &moduleSummaryIndex,
                                /* generate ThinLTO hash */ true);
@@ -413,7 +383,7 @@ void writeModule(llvm::Module *m, const char *filename) {
 
   // write LLVM IR
   if (global.params.output_ll) {
-    const auto llpath = replaceExtensionWith(global.ll_ext);
+    const auto llpath = replaceExtensionWith(global.ll_ext, filename);
     Logger::println("Writing LLVM IR to: %s\n", llpath.c_str());
     std::error_code errinfo;
     llvm::raw_fd_ostream aos(llpath.c_str(), errinfo, llvm::sys::fs::F_None);
@@ -435,7 +405,7 @@ void writeModule(llvm::Module *m, const char *filename) {
       llvm::sys::fs::createUniqueFile("ldc-%%%%%%%.s", buffer);
       spath = {buffer.data(), buffer.size()};
     } else {
-      spath = replaceExtensionWith(global.s_ext);
+      spath = replaceExtensionWith(global.s_ext, filename);
     }
 
     Logger::println("Writing asm to: %s\n", spath.c_str());
