@@ -38,12 +38,15 @@ std::vector<const char *> getFullArgs(const char *tool,
                                       bool printVerbose);
 
 int executeToolAndWait(const std::string &tool,
-                       std::vector<std::string> const &args,
+                       const std::vector<std::string> &args,
                        bool verbose = false);
 
 #ifdef _WIN32
 
 namespace windows {
+// Returns true if a usable MSVC installation is available.
+bool isMsvcAvailable();
+
 struct MsvcEnvironmentScope {
   // Tries to set up the MSVC environment variables for the current process and
   // returns true if successful. The original environment is restored on
