@@ -65,12 +65,10 @@ DLValue *DtoAAIndex(Loc &loc, Type *type, DValue *aa, DValue *key,
     LLValue *castedAATI = DtoBitCast(rawAATI, funcTy->getParamType(1));
     LLValue *valsize = DtoConstSize_t(getTypeAllocSize(DtoType(type)));
     ret = gIR->CreateCallOrInvoke(func, aaval, castedAATI, valsize, pkey,
-                                  "aa.index")
-              .getInstruction();
+                                  "aa.index");
   } else {
     LLValue *keyti = to_keyti(aa, funcTy->getParamType(1));
-    ret = gIR->CreateCallOrInvoke(func, aaval, keyti, pkey, "aa.index")
-              .getInstruction();
+    ret = gIR->CreateCallOrInvoke(func, aaval, keyti, pkey, "aa.index");
   }
 
   // cast return value
@@ -91,12 +89,12 @@ DLValue *DtoAAIndex(Loc &loc, Type *type, DValue *aa, DValue *key,
 
     // set up failbb to call the array bounds error runtime function
 
-    gIR->scope() = IRScope(failbb);
+    gIR->ir->SetInsertPoint(failbb);
 
     DtoBoundsCheckFailCall(gIR, loc);
 
     // if ok, proceed in okbb
-    gIR->scope() = IRScope(okbb);
+    gIR->ir->SetInsertPoint(okbb);
   }
   return new DLValue(type, ret);
 }
@@ -134,8 +132,7 @@ DValue *DtoAAIn(Loc &loc, Type *type, DValue *aa, DValue *key) {
   pkey = DtoBitCast(pkey, getVoidPtrType());
 
   // call runtime
-  LLValue *ret = gIR->CreateCallOrInvoke(func, aaval, keyti, pkey, "aa.in")
-                     .getInstruction();
+  LLValue *ret = gIR->CreateCallOrInvoke(func, aaval, keyti, pkey, "aa.in");
 
   // cast return value
   LLType *targettype = DtoType(type);
@@ -179,9 +176,9 @@ DValue *DtoAARemove(Loc &loc, DValue *aa, DValue *key) {
   pkey = DtoBitCast(pkey, funcTy->getParamType(2));
 
   // call runtime
-  LLCallSite call = gIR->CreateCallOrInvoke(func, aaval, keyti, pkey);
+  LLValue *res = gIR->CreateCallOrInvoke(func, aaval, keyti, pkey);
 
-  return new DImValue(Type::tbool, call.getInstruction());
+  return new DImValue(Type::tbool, res);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -197,8 +194,7 @@ LLValue *DtoAAEquals(Loc &loc, TOK op, DValue *l, DValue *r) {
   LLValue *abval = DtoBitCast(DtoRVal(r), funcTy->getParamType(2));
   LLValue *aaTypeInfo = DtoTypeInfoOf(t);
   LLValue *res =
-      gIR->CreateCallOrInvoke(func, aaTypeInfo, aaval, abval, "aaEqRes")
-          .getInstruction();
+      gIR->CreateCallOrInvoke(func, aaTypeInfo, aaval, abval, "aaEqRes");
 
   const auto predicate = eqTokToICmpPred(op, /* invert = */ true);
   res = gIR->ir->CreateICmp(predicate, res, DtoConstInt(0));
