@@ -11,12 +11,10 @@
 //
 //===----------------------------------------------------------------------===//
 
-#ifndef LDC_GEN_AA_H
-#define LDC_GEN_AA_H
+#pragma once
 
-#include "tokens.h"
+#include "dmd/tokens.h"
 
-enum TOK;
 class DValue;
 class DLValue;
 struct Loc;
@@ -29,5 +27,3 @@ DLValue *DtoAAIndex(Loc &loc, Type *type, DValue *aa, DValue *key, bool lvalue);
 DValue *DtoAAIn(Loc &loc, Type *type, DValue *aa, DValue *key);
 DValue *DtoAARemove(Loc &loc, DValue *aa, DValue *key);
 llvm::Value *DtoAAEquals(Loc &loc, TOK op, DValue *l, DValue *r);
-
-#endif // LDC_GEN_AA_H
