@@ -99,7 +99,7 @@ public:
   }
 
   bool returnInArg(TypeFunction *tf, bool needsThis) override {
-    if (tf->isref)
+    if (tf->isref())
       return false;
 
     Type *rt = tf->next->toBasetype();
