@@ -25,10 +25,6 @@ class TypeClass;
 /// Resolves the llvm type for a class declaration
 void DtoResolveClass(ClassDeclaration *cd);
 
-/// Builds the initializer of cd's ClassInfo.
-/// FIXME: this should be put into IrStruct and eventually IrClass.
-llvm::Constant *DtoDefineClassInfo(ClassDeclaration *cd);
-
 DValue *DtoNewClass(Loc &loc, TypeClass *type, NewExp *newexp);
 void DtoInitClass(TypeClass *tc, llvm::Value *dst);
 void DtoFinalizeClass(Loc &loc, llvm::Value *inst);
@@ -39,5 +35,4 @@ DValue *DtoDynamicCastObject(Loc &loc, DValue *val, Type *to);
 
 DValue *DtoDynamicCastInterface(Loc &loc, DValue *val, Type *to);
 
-llvm::Value *DtoVirtualFunctionPointer(DValue *inst, FuncDeclaration *fdecl,
-                                       const char *name);
+llvm::Value *DtoVirtualFunctionPointer(DValue *inst, FuncDeclaration *fdecl);
