@@ -9,7 +9,6 @@
 
 #include "gen/coverage.h"
 
-#include "dmd/mars.h"
 #include "dmd/module.h"
 #include "gen/irstate.h"
 #include "gen/logger.h"
@@ -21,7 +20,7 @@ void emitCoverageLinecountInc(Loc &loc) {
   // module
   // (for example, 'inlined' methods from other source files should be skipped).
   if (!global.params.cov || !loc.linnum || !loc.filename || !m->d_cover_data ||
-      strcmp(m->srcfile->name.toChars(), loc.filename) != 0) {
+      strcmp(m->srcfile.toChars(), loc.filename) != 0) {
     return;
   }
 
