@@ -19,9 +19,16 @@
 #include "gen/dcompute/target.h"
 #include "gen/llvmhelpers.h"
 #include "gen/runtime.h"
-#include <string>
+#include "ir/irtypestruct.h"
+
 
 void DComputeTarget::doCodeGen(Module *m) {
+  // Reset any generated type info for dcompute types.
+  // The ll types get generated when the host code gets
+  // gen'd which means the address space info is not
+  // properly set.
+  IrTypeStruct::resetDComputeTypes();
+
   // process module members
   for (unsigned k = 0; k < m->members->length; k++) {
     Dsymbol *dsym = (*m->members)[k];
