@@ -102,7 +102,7 @@ public:
       }
 
       // Emit TypeInfo.
-      IrAggr *ir = getIrAggr(decl);
+      IrClass *ir = getIrAggr(decl);
       if (!ir->suppressTypeInfo() && !isSpeculativeType(decl->type)) {
         llvm::GlobalVariable *interfaceZ = ir->getClassInfoSymbol();
         defineGlobal(interfaceZ, ir->getClassInfoInit(), decl);
@@ -141,7 +141,7 @@ public:
     // Skip __initZ and typeinfo for @compute device code.
     // TODO: support global variables and thus __initZ
     if (!irs->dcomputetarget) {
-      IrAggr *ir = getIrAggr(decl);
+      IrStruct *ir = getIrAggr(decl);
 
       // Define the __initZ symbol.
       if (!decl->zeroInit) {
@@ -196,7 +196,7 @@ public:
         m->accept(this);
       }
 
-      IrAggr *ir = getIrAggr(decl);
+      IrClass *ir = getIrAggr(decl);
 
       auto &initZ = ir->getInitSymbol();
       auto initGlobal = llvm::cast<LLGlobalVariable>(initZ);
@@ -371,28 +371,19 @@ public:
       return;
     }
 
-    // Force codegen if this is a templated function with pragma(inline, true).
-    if ((decl->members->length == 1) &&
-        ((*decl->members)[0]->isFuncDeclaration()) &&
-        ((*decl->members)[0]->isFuncDeclaration()->inlining == PINLINEalways)) {
-      Logger::println("needsCodegen() == false, but function is marked with "
-                      "pragma(inline, true), so it really does need "
-                      "codegen.");
-    } else {
-      // FIXME: This is #673 all over again.
-      if (!decl->needsCodegen()) {
-        Logger::println("Does not need codegen, skipping.");
-        return;
-      }
+    // FIXME: This is #673 all over again.
+    if (!decl->needsCodegen()) {
+      Logger::println("Does not need codegen, skipping.");
+      return;
+    }
 
-      if (irs->dcomputetarget && (decl->tempdecl == Type::rtinfo ||
-                                  decl->tempdecl == Type::rtinfoImpl)) {
-        // Emitting object.RTInfo(Impl) template instantiations in dcompute
-        // modules would require dcompute support for global variables.
-        Logger::println("Skipping object.RTInfo(Impl) template instantiations "
-                        "in dcompute modules.");
-        return;
-      }
+    if (irs->dcomputetarget && (decl->tempdecl == Type::rtinfo ||
+                                decl->tempdecl == Type::rtinfoImpl)) {
+      // Emitting object.RTInfo(Impl) template instantiations in dcompute
+      // modules would require dcompute support for global variables.
+      Logger::println("Skipping object.RTInfo(Impl) template instantiations "
+                      "in dcompute modules.");
+      return;
     }
 
     for (auto &m : *decl->members) {
@@ -513,14 +504,7 @@ public:
 
   void visit(TypeInfoDeclaration *decl) override {
     if (!irs->dcomputetarget)
-      TypeInfoDeclaration_codegen(decl, irs);
-  }
-
-  //////////////////////////////////////////////////////////////////////////
-
-  void visit(TypeInfoClassDeclaration *decl) override {
-    if (!irs->dcomputetarget)
-      TypeInfoClassDeclaration_codegen(decl, irs);
+      TypeInfoDeclaration_codegen(decl);
   }
 };
 
