@@ -8,7 +8,8 @@
 //===----------------------------------------------------------------------===//
 
 #include "gen/dvalue.h"
-#include "declaration.h"
+
+#include "dmd/declaration.h"
 #include "gen/irstate.h"
 #include "gen/llvm.h"
 #include "gen/llvmhelpers.h"
@@ -68,6 +69,7 @@ DImValue::DImValue(Type *t, llvm::Value *v) : DRValue(t, v) {
   // equality check.
   assert(t->toBasetype()->ty == Tfunction ||
          stripAddrSpaces(v->getType()) == DtoType(t));
+  assert(t->toBasetype()->ty != Tarray && "use DSliceValue for dynamic arrays");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -78,19 +80,27 @@ DConstValue::DConstValue(Type *t, LLConstant *con) : DRValue(t, con) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-DSliceValue::DSliceValue(Type *t, LLValue *pair) : DRValue(t, pair) {
+DSliceValue::DSliceValue(Type *t, LLValue *pair, LLValue *length, LLValue *ptr)
+    : DRValue(t, pair), length(length), ptr(ptr) {
   assert(t->toBasetype()->ty == Tarray);
   // v may be an addrspace qualified pointer so strip it before doing a pointer
   // equality check.
   assert(stripAddrSpaces(pair->getType()) == DtoType(t));
 }
 
+DSliceValue::DSliceValue(Type *t, LLValue *pair)
+    : DSliceValue(t, pair, nullptr, nullptr) {}
+
 DSliceValue::DSliceValue(Type *t, LLValue *length, LLValue *ptr)
-    : DSliceValue(t, DtoAggrPair(length, ptr)) {}
+    : DSliceValue(t, DtoAggrPair(length, ptr), length, ptr) {}
 
-LLValue *DSliceValue::getLength() { return DtoExtractValue(val, 0, ".len"); }
+LLValue *DSliceValue::getLength() {
+  return length ? length : DtoExtractValue(val, 0, ".len");
+}
 
-LLValue *DSliceValue::getPtr() { return DtoExtractValue(val, 1, ".ptr"); }
+LLValue *DSliceValue::getPtr() {
+  return ptr ? ptr : DtoExtractValue(val, 1, ".ptr");
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -121,7 +131,9 @@ DRValue *DLValue::getRVal() {
   }
 
   LLValue *rval = DtoLoad(val);
-  if (type->toBasetype()->ty == Tbool) {
+
+  const auto ty = type->toBasetype()->ty;
+  if (ty == Tbool) {
     assert(rval->getType() == llvm::Type::getInt8Ty(gIR->context()));
 
     if (isOptimizationEnabled()) {
@@ -134,6 +146,8 @@ DRValue *DLValue::getRVal() {
 
     // truncate to i1
     rval = gIR->ir->CreateTrunc(rval, llvm::Type::getInt1Ty(gIR->context()));
+  } else if (ty == Tarray) {
+    return new DSliceValue(type, rval);
   }
 
   return new DImValue(type, rval);
