@@ -94,6 +94,7 @@ DValue *DtoCastInt(Loc &loc, DValue *val, Type *to);
 DValue *DtoCastPtr(Loc &loc, DValue *val, Type *to);
 DValue *DtoCastFloat(Loc &loc, DValue *val, Type *to);
 DValue *DtoCastDelegate(Loc &loc, DValue *val, Type *to);
+DValue *DtoCastVector(Loc &loc, DValue *val, Type *to);
 DValue *DtoCast(Loc &loc, DValue *val, Type *to);
 
 // return the same val as passed in, modified to the target type, if possible,
@@ -146,10 +147,10 @@ unsigned getFieldGEPIndex(AggregateDeclaration *ad, VarDeclaration *vd);
 DValue *DtoInlineAsmExpr(Loc &loc, FuncDeclaration *fd, Expressions *arguments,
                          LLValue *sretPointer = nullptr);
 ///
-llvm::CallInst *DtoInlineAsmExpr(
-    const Loc &loc, llvm::StringRef code, llvm::StringRef constraints,
-    llvm::ArrayRef<llvm::Value *> indirectOutputLVals,
-    llvm::ArrayRef<Expression *> arguments, llvm::Type *returnType);
+llvm::CallInst *DtoInlineAsmExpr(const Loc &loc, llvm::StringRef code,
+                                 llvm::StringRef constraints,
+                                 llvm::ArrayRef<llvm::Value *> operands,
+                                 llvm::Type *returnType);
 
 /// Returns the size the LLVM type for a member variable of the given type will
 /// take up in a struct (in bytes). This does not include padding in any way.
@@ -240,11 +241,6 @@ LLConstant *toConstantArray(LLType *ct, LLArrayType *at, T *str, size_t len,
   return LLConstantArray::get(at, vals);
 }
 
-/// Returns the cache for string literals of the given character type (for the
-/// current IRState).
-llvm::StringMap<llvm::GlobalVariable *> *
-stringLiteralCacheForType(Type *charType);
-
 llvm::Constant *buildStringLiteralConstant(StringExp *se, bool zeroTerm);
 
 /// Tries to declare an LLVM global. If a variable with the same mangled name
