@@ -10,6 +10,7 @@
 #include "gen/nested.h"
 
 #include "dmd/errors.h"
+#include "dmd/ldcbindings.h"
 #include "dmd/target.h"
 #include "gen/dvalue.h"
 #include "gen/funcgenstate.h"
@@ -65,16 +66,16 @@ DValue *DtoNestedVariable(Loc &loc, Type *astype, VarDeclaration *vd,
   if (currentCtx) {
     Logger::println("Using own nested context of current function");
     ctx = currentCtx;
-  } else if (irfunc->decl->isMember2()) {
+  } else if (AggregateDeclaration *ad = irfunc->decl->isMember2()) {
     Logger::println(
         "Current function is member of nested class, loading vthis");
-
-    AggregateDeclaration *cd = irfunc->decl->isMember2();
-    LLValue *val = irfunc->thisArg;
-    if (cd->isClassDeclaration()) {
-      val = DtoLoad(val);
+    LLValue *val =
+        ad->isClassDeclaration() ? DtoLoad(irfunc->thisArg) : irfunc->thisArg;
+    for (; ad; ad = ad->toParent2()->isAggregateDeclaration()) {
+      assert(ad->vthis);
+      val = DtoLoad(DtoGEP(val, 0, getVthisIdx(ad), ".vthis"));
     }
-    ctx = DtoLoad(DtoGEP(val, 0, getVthisIdx(cd), ".vthis"));
+    ctx = val;
     skipDIDeclaration = true;
   } else {
     Logger::println("Regular nested function, using context arg");
@@ -170,9 +171,6 @@ DValue *DtoNestedVariable(Loc &loc, Type *astype, VarDeclaration *vd,
   }
 
   if (!skipDIDeclaration && global.params.symdebug) {
-#if LDC_LLVM_VER < 500
-    gIR->DBuilder.OpDeref(dwarfAddrOps);
-#endif
     gIR->DBuilder.EmitLocalVariable(ctx, vd, nullptr, false,
                                     /*forceAsLocal=*/true, false, dwarfAddrOps);
   }
@@ -391,21 +389,21 @@ static void DtoCreateNestedContextType(FuncDeclaration *fd) {
       builder.alignCurrentOffset(alignment);
     }
 
-    IrLocal &irLocal = *getIrLocal(vd, true);
+    const bool isParam = vd->isParameter();
+
+    IrLocal &irLocal =
+        *(isParam ? getIrParameter(vd, true) : getIrLocal(vd, true));
     irLocal.nestedIndex = builder.currentFieldIndex();
     irLocal.nestedDepth = depth;
 
     LLType *t = nullptr;
     if (vd->isRef() || vd->isOut()) {
       t = DtoType(vd->type->pointerTo());
-    } else if (vd->isParameter() && (vd->storage_class & STClazy)) {
-      // The LL type is a delegate (LL struct).
-      // Depending on the used TargetABI, the LL parameter is either a struct or
-      // a pointer to a struct (`byval` attribute, IndirectByvalRewrite).
-      t = getIrParameter(vd)->value->getType();
-      if (t->isPointerTy())
-        t = t->getPointerElementType();
-      assert(t->isStructTy());
+    } else if (isParam && (vd->storage_class & STClazy)) {
+      // the type is a delegate (LL struct)
+      Type *dt = TypeFunction::create(nullptr, vd->type, VARARGnone, LINKd);
+      dt = createTypeDelegate(dt);
+      t = DtoType(dt);
     } else {
       t = DtoMemType(vd->type);
     }
@@ -457,17 +455,13 @@ void DtoCreateNestedContext(FuncGenState &funcGen) {
       LLValue *src = irFunc.nestArg;
       if (!src) {
         assert(irFunc.thisArg);
-        assert(fd->isMember2());
-        LLValue *thisval = DtoLoad(irFunc.thisArg);
-        AggregateDeclaration *cd = fd->isMember2();
-        assert(cd);
-        assert(cd->vthis);
+        AggregateDeclaration *ad = fd->isMember2();
+        assert(ad);
+        assert(ad->vthis);
+        LLValue *thisptr =
+            ad->isClassDeclaration() ? DtoLoad(irFunc.thisArg) : irFunc.thisArg;
         IF_LOG Logger::println("Indexing to 'this'");
-        if (cd->isStructDeclaration()) {
-          src = DtoExtractValue(thisval, getVthisIdx(cd), ".vthis");
-        } else {
-          src = DtoLoad(DtoGEP(thisval, 0, getVthisIdx(cd), ".vthis"));
-        }
+        src = DtoLoad(DtoGEP(thisptr, 0, getVthisIdx(ad), ".vthis"));
       }
       if (depth > 1) {
         src = DtoBitCast(src, getVoidPtrType());
@@ -502,7 +496,8 @@ void DtoCreateNestedContext(FuncGenState &funcGen) {
         assert(parm->value->getType()->isPointerTy());
 
         if (vd->isRef() || vd->isOut()) {
-          Logger::println("Captured by reference, copying pointer to nested frame");
+          Logger::println(
+              "Captured by reference, copying pointer to nested frame");
           DtoAlignedStore(parm->value, gep);
           // pass GEP as reference lvalue to EmitLocalVariable()
         } else {
@@ -522,11 +517,6 @@ void DtoCreateNestedContext(FuncGenState &funcGen) {
 
       if (global.params.symdebug) {
         LLSmallVector<int64_t, 1> dwarfAddrOps;
-#if LDC_LLVM_VER < 500
-        // Because we are passing a GEP instead of an alloca to
-        // llvm.dbg.declare, we have to make the address dereference explicit.
-        gIR->DBuilder.OpDeref(dwarfAddrOps);
-#endif
         gIR->DBuilder.EmitLocalVariable(gep, vd, nullptr, false, false, false,
                                         dwarfAddrOps);
       }
