@@ -187,12 +187,10 @@ namespace {
 void pushVarDtorCleanup(IRState *p, VarDeclaration *vd) {
   llvm::BasicBlock *beginBB = p->insertBB(llvm::Twine("dtor.") + vd->toChars());
 
-  // TODO: Clean this up with push/pop insertion point methods.
-  IRScope oldScope = p->scope();
-  p->scope() = IRScope(beginBB);
+  const auto savedInsertPoint = p->saveInsertPoint();
+  p->ir->SetInsertPoint(beginBB);
   toElemDtor(vd->edtor);
   p->funcGen().scopes.pushCleanup(beginBB, p->scopebb());
-  p->scope() = oldScope;
 }
 }
 
@@ -253,7 +251,7 @@ public:
       llvm::BasicBlock *endbb = p->insertBB("toElem.success");
       p->funcGen().scopes.runCleanups(initialCleanupScope, endbb);
       p->funcGen().scopes.popCleanups(initialCleanupScope);
-      p->scope() = IRScope(endbb);
+      p->ir->SetInsertPoint(endbb);
 
       destructTemporaries = false;
     }
@@ -383,31 +381,9 @@ public:
 
     Type *dtype = e->type->toBasetype();
     Type *cty = dtype->nextOf()->toBasetype();
-
     LLType *ct = DtoMemType(cty);
 
-    llvm::StringMap<llvm::GlobalVariable *> *stringLiteralCache =
-        stringLiteralCacheForType(cty);
-    LLConstant *_init = buildStringLiteralConstant(e, true);
-    const auto at = _init->getType();
-
-    llvm::StringRef key(e->toChars());
-    llvm::GlobalVariable *gvar =
-        (stringLiteralCache->find(key) == stringLiteralCache->end())
-            ? nullptr
-            : (*stringLiteralCache)[key];
-    if (gvar == nullptr) {
-      llvm::GlobalValue::LinkageTypes _linkage =
-          llvm::GlobalValue::PrivateLinkage;
-      IF_LOG {
-        Logger::cout() << "type: " << *at << '\n';
-        Logger::cout() << "init: " << *_init << '\n';
-      }
-      gvar = new llvm::GlobalVariable(gIR->module, at, true, _linkage, _init,
-                                      ".str");
-      gvar->setUnnamedAddr(llvm::GlobalValue::UnnamedAddr::Global);
-      (*stringLiteralCache)[key] = gvar;
-    }
+    llvm::GlobalVariable *gvar = p->getCachedStringLiteral(e);
 
     llvm::ConstantInt *zero =
         LLConstantInt::get(LLType::getInt32Ty(gIR->context()), 0, false);
@@ -456,7 +432,8 @@ public:
     // Can't just override ConstructExp::toElem because not all TOKconstruct
     // operations are actually instances of ConstructExp... Long live the DMD
     // coding style!
-    if (e->memset & referenceInit) {
+    if (static_cast<int>(e->memset) &
+        static_cast<int>(MemorySet::referenceInit)) {
       assert(e->op == TOKconstruct || e->op == TOKblit);
       auto ve = e->e1->isVarExp();
       assert(ve);
@@ -732,7 +709,6 @@ public:
       assert(dve);
       FuncDeclaration *fdecl = dve->var->isFuncDeclaration();
       assert(fdecl);
-      DtoDeclareFunction(fdecl);
       Expression *thisExp = dve->e1;
       LLValue *thisArg = thisExp->type->toBasetype()->ty == Tclass
                              ? DtoRVal(thisExp)
@@ -890,7 +866,6 @@ public:
       // Logger::println("FuncDeclaration");
       FuncDeclaration *fd = fv->func;
       assert(fd);
-      DtoResolveFunction(fd);
       result = new DFuncValue(fd, DtoCallee(fd));
       return;
     }
@@ -984,8 +959,6 @@ public:
       // Logger::cout() << "mem: " << *arrptr << '\n';
       result = new DLValue(e->type, DtoBitCast(arrptr, DtoPtrToType(e->type)));
     } else if (FuncDeclaration *fdecl = e->var->isFuncDeclaration()) {
-      DtoResolveFunction(fdecl);
-
       // This is a bit more convoluted than it would need to be, because it
       // has to take templated interface methods into account, for which
       // isFinalFunc is not necessarily true.
@@ -998,7 +971,8 @@ public:
       // Get the actual function value to call.
       LLValue *funcval = nullptr;
       if (nonFinal) {
-        funcval = DtoVirtualFunctionPointer(l, fdecl, e->toChars());
+        DtoResolveFunction(fdecl);
+        funcval = DtoVirtualFunctionPointer(l, fdecl);
       } else {
         funcval = DtoCallee(fdecl);
       }
@@ -1170,10 +1144,10 @@ public:
 
         p->ir->CreateCondBr(okCond, okbb, failbb);
 
-        p->scope() = IRScope(failbb);
+        p->ir->SetInsertPoint(failbb);
         DtoBoundsCheckFailCall(p, e->loc);
 
-        p->scope() = IRScope(okbb);
+        p->ir->SetInsertPoint(okbb);
       }
 
       // offset by lower
@@ -1292,19 +1266,19 @@ public:
                                                    lfptr, rfptr, ".fptreqcmp");
         llvm::BranchInst::Create(fptreq, fptrneq, fptreqcmp, p->scopebb());
 
-        p->scope() = IRScope(fptreq);
+        p->ir->SetInsertPoint(fptreq);
         llvm::Value *lctx = p->ir->CreateExtractValue(lhs, 0, ".lctx");
         llvm::Value *rctx = p->ir->CreateExtractValue(rhs, 0, ".rctx");
         llvm::Value *ctxcmp =
             p->ir->CreateICmp(icmpPred, lctx, rctx, ".ctxcmp");
         llvm::BranchInst::Create(dgcmpend, p->scopebb());
 
-        p->scope() = IRScope(fptrneq);
+        p->ir->SetInsertPoint(fptrneq);
         llvm::Value *fptrcmp =
             p->ir->CreateICmp(icmpPred, lfptr, rfptr, ".fptrcmp");
         llvm::BranchInst::Create(dgcmpend, p->scopebb());
 
-        p->scope() = IRScope(dgcmpend);
+        p->ir->SetInsertPoint(dgcmpend);
         llvm::PHINode *phi = p->ir->CreatePHI(ctxcmp->getType(), 2, ".dgcmp");
         phi->addIncoming(ctxcmp, fptreq);
         phi->addIncoming(fptrcmp, fptrneq);
@@ -1507,7 +1481,6 @@ public:
       LLValue *mem = nullptr;
       if (e->allocator) {
         // custom allocator
-        DtoResolveFunction(e->allocator);
         DFuncValue dfn(e->allocator, DtoCallee(e->allocator));
         DValue *res = DtoCallFunction(e->loc, nullptr, &dfn, e->newargs);
         mem = DtoBitCast(DtoRVal(res), DtoType(ntype->pointerTo()),
@@ -1536,7 +1509,6 @@ public:
 
           IF_LOG Logger::println("Calling constructor");
           assert(e->arguments != NULL);
-          DtoResolveFunction(e->member);
           DFuncValue dfn(e->member, DtoCallee(e->member), mem);
           DtoCallFunction(e->loc, ts, &dfn, e->arguments);
         }
@@ -1682,31 +1654,37 @@ public:
     // assign branch weights to this branch instruction.
 
     // failed: call assert runtime function
-    p->scope() = IRScope(failedbb);
+    p->ir->SetInsertPoint(failedbb);
 
-    /* DMD Bugzilla 8360: If the condition is evaluated to true,
-     * msg is not evaluated at all. So should use toElemDtor()
-     * instead of toElem().
-     */
-    DValue *const msg = e->msg ? toElemDtor(e->msg) : nullptr;
-    Module *const module = p->func()->decl->getModule();
-    if (global.params.checkAction == CHECKACTION_C) {
-      const auto cMsg =
-          msg ? DtoArrayPtr(msg) // assuming `msg` is null-terminated, like DMD
-              : DtoConstCString(e->e1->toChars());
-      DtoCAssert(module, e->e1->loc, cMsg);
+    if (global.params.checkAction == CHECKACTION_halt) {
+      p->ir->CreateCall(GET_INTRINSIC_DECL(trap), {});
+      p->ir->CreateUnreachable();
     } else {
-      DtoAssert(module, e->loc, msg);
+      /* DMD Bugzilla 8360: If the condition is evaluated to true,
+       * msg is not evaluated at all. So should use toElemDtor()
+       * instead of toElem().
+       */
+      DValue *msg = e->msg ? toElemDtor(e->msg) : nullptr;
+      Module *module = p->func()->decl->getModule();
+      if (global.params.checkAction == CHECKACTION_C) {
+        LLValue *cMsg =
+            msg ? DtoArrayPtr(
+                      msg) // assuming `msg` is null-terminated, like DMD
+                : DtoConstCString(e->e1->toChars());
+        DtoCAssert(module, e->e1->loc, cMsg);
+      } else {
+        DtoAssert(module, e->loc, msg);
+      }
     }
 
     // passed:
-    p->scope() = IRScope(passedbb);
+    p->ir->SetInsertPoint(passedbb);
 
     // class/struct invariants
     if (global.params.useInvariants != CHECKENABLEon)
       return;
-    if (condty->ty == Tclass) {
-      const auto sym = static_cast<TypeClass *>(condty)->sym;
+    if (auto tc = condty->isTypeClass()) {
+      const auto sym = tc->sym;
       if (sym->isInterfaceDeclaration() || sym->isCPPclass())
         return;
 
@@ -1728,7 +1706,6 @@ public:
 
       Logger::print("calling struct invariant");
 
-      DtoResolveFunction(invDecl);
       DFuncValue invFunc(invDecl, DtoCallee(invDecl), DtoRVal(cond));
       DtoCallFunction(e->loc, nullptr, &invFunc, nullptr);
     }
@@ -1786,7 +1763,7 @@ public:
     p->ir->CreateCondBr(ubool, isAndAnd ? rhsBB : endBB,
                         isAndAnd ? endBB : rhsBB, branchweights);
 
-    p->scope() = IRScope(rhsBB);
+    p->ir->SetInsertPoint(rhsBB);
     PGO.emitCounterIncrement(e);
     emitCoverageLinecountInc(e->e2->loc);
     DValue *v = toElemDtor(e->e2);
@@ -1798,7 +1775,7 @@ public:
 
     llvm::BasicBlock *newblock = p->scopebb();
     llvm::BranchInst::Create(endBB, p->scopebb());
-    p->scope() = IRScope(endBB);
+    p->ir->SetInsertPoint(endBB);
 
     // DMD allows stuff like `x == 0 && assert(false)`
     if (e->type->toBasetype()->ty == Tvoid) {
@@ -1843,7 +1820,7 @@ public:
     // this is sensible, since someone might goto behind the assert
     // and prevents compiler errors if a terminator follows the assert
     llvm::BasicBlock *bb = p->insertBB("afterhalt");
-    p->scope() = IRScope(bb);
+    p->ir->SetInsertPoint(bb);
   }
 
   //////////////////////////////////////////////////////////////////////////////
@@ -1884,7 +1861,7 @@ public:
 
     if (e->e1->op != TOKsuper && e->e1->op != TOKdottype &&
         e->func->isVirtual() && !e->func->isFinalFunc()) {
-      castfptr = DtoVirtualFunctionPointer(u, e->func, e->toChars());
+      castfptr = DtoVirtualFunctionPointer(u, e->func);
     } else if (e->func->isAbstract()) {
       llvm_unreachable("Delegate to abstract method not implemented.");
     } else if (e->func->toParent()->isInterfaceDeclaration()) {
@@ -2025,7 +2002,7 @@ public:
     auto branchweights = PGO.createProfileWeights(truecount, falsecount);
     p->ir->CreateCondBr(cond_val, condtrue, condfalse, branchweights);
 
-    p->scope() = IRScope(condtrue);
+    p->ir->SetInsertPoint(condtrue);
     PGO.emitCounterIncrement(e);
     DValue *u = toElem(e->e1);
     if (retPtr) {
@@ -2034,7 +2011,7 @@ public:
     }
     llvm::BranchInst::Create(condend, p->scopebb());
 
-    p->scope() = IRScope(condfalse);
+    p->ir->SetInsertPoint(condfalse);
     DValue *v = toElem(e->e2);
     if (retPtr) {
       LLValue *lval = makeLValue(e->loc, v);
@@ -2042,7 +2019,7 @@ public:
     }
     llvm::BranchInst::Create(condend, p->scopebb());
 
-    p->scope() = IRScope(condend);
+    p->ir->SetInsertPoint(condend);
     if (retPtr)
       result = new DSpecialRefValue(e->type, retPtr);
   }
@@ -2168,7 +2145,7 @@ public:
       DtoDeclareFunction(fd);
       assert(!fd->isNested());
     }
-    assert(DtoCallee(fd));
+    assert(DtoCallee(fd, false));
   }
 
   //////////////////////////////////////////////////////////////////////////////
@@ -2182,41 +2159,19 @@ public:
     assert(fd);
 
     genFuncLiteral(fd, e);
+    LLFunction *callee = DtoCallee(fd, false);
 
     if (fd->isNested()) {
       LLType *dgty = DtoType(e->type);
 
-      LLValue *cval;
-      auto &funcGen = p->funcGen();
-      auto &irfn = funcGen.irFunc;
-      if (funcGen.nestedVar && fd->toParent2() == irfn.decl) {
-        // We check fd->toParent2() because a frame allocated in one
-        // function cannot be used for a delegate created in another
-        // function. Happens with anonymous functions.
-        cval = funcGen.nestedVar;
-      } else if (irfn.nestArg) {
-        cval = irfn.nestArg;
-      } else if (irfn.thisArg) {
-        AggregateDeclaration *ad = irfn.decl->isMember2();
-        if (!ad || !ad->vthis) {
-          cval = getNullPtr(getVoidPtrType());
-        } else {
-          cval =
-              ad->isClassDeclaration() ? DtoLoad(irfn.thisArg) : irfn.thisArg;
-          cval = DtoLoad(
-              DtoGEP(cval, 0, getFieldGEPIndex(ad, ad->vthis), ".vthis"));
-        }
-      } else {
-        cval = getNullPtr(getVoidPtrType());
-      }
+      LLValue *cval = DtoNestedContext(e->loc, fd);
       cval = DtoBitCast(cval, dgty->getContainedType(0));
 
-      LLValue *castfptr = DtoBitCast(DtoCallee(fd), dgty->getContainedType(1));
+      LLValue *castfptr = DtoBitCast(callee, dgty->getContainedType(1));
 
       result = new DImValue(e->type, DtoAggrPair(cval, castfptr, ".func"));
-
     } else {
-      result = new DFuncValue(e->type, fd, DtoCallee(fd));
+      result = new DFuncValue(e->type, fd, callee);
     }
   }
 
@@ -2472,8 +2427,7 @@ public:
       LLValue *valuesArray = DtoAggrPaint(slice, funcTy->getParamType(2));
 
       LLValue *aa = gIR->CreateCallOrInvoke(func, aaTypeInfo, keysArray,
-                                            valuesArray, "aa")
-                        .getInstruction();
+                                            valuesArray, "aa");
       if (basetype->ty != Taarray) {
         LLValue *tmp = DtoAlloca(e->type, "aaliteral");
         DtoStore(aa, DtoGEP(tmp, 0u, 0));
@@ -2593,8 +2547,8 @@ public:
   //////////////////////////////////////////////////////////////////////////////
 
   static DLValue *emitVector(VectorExp *e, LLValue *dstMem) {
-    TypeVector *type = static_cast<TypeVector *>(e->to->toBasetype());
-    assert(e->type->ty == Tvector);
+    TypeVector *type = e->to->toBasetype()->isTypeVector();
+    assert(type);
 
     const unsigned N = e->dim;
 
@@ -2602,9 +2556,15 @@ public:
     if (elementType->ty == Tvoid)
       elementType = Type::tuns8;
 
-    // Array literals are assigned element-wise, other expressions are cast and
-    // splat across the vector elements. This is what DMD does.
+    const auto getCastElement = [e, elementType](DValue *element) {
+      return DtoRVal(DtoCast(e->loc, element, elementType));
+    };
+
+    Type *tsrc = e->e1->type->toBasetype();
     if (auto lit = e->e1->isArrayLiteralExp()) {
+      // Optimization for array literals: check for a fully static literal and
+      // store a vector constant in that case, otherwise emplace element-wise
+      // into destination memory.
       Logger::println("array literal expression");
       assert(lit->elements->length == N &&
              "Array literal vector initializer "
@@ -2616,7 +2576,7 @@ public:
       llElementConstants.reserve(N);
       for (unsigned i = 0; i < N; ++i) {
         DValue *val = toElem(indexArrayLiteral(lit, i));
-        LLValue *llVal = DtoRVal(DtoCast(e->loc, val, elementType));
+        LLValue *llVal = getCastElement(val);
         llElements.push_back(llVal);
         if (auto llConstant = isaConstant(llVal))
           llElementConstants.push_back(llConstant);
@@ -2630,12 +2590,44 @@ public:
           DtoStore(llElements[i], DtoGEP(dstMem, 0, i));
         }
       }
+    } else if (tsrc->ty == Tarray || tsrc->ty == Tsarray) {
+      // Arrays: prefer a memcpy if the LL element types match, otherwise cast
+      // and store element-wise.
+      if (auto ts = tsrc->isTypeSArray()) {
+        Logger::println("static array expression");
+        assert(ts->dim->toInteger() == N &&
+               "Static array vector initializer length mismatch, should have "
+               "been handled in frontend.");
+      } else {
+        // TODO: bounds check?
+        Logger::println("dynamic array expression, assume matching length");
+      }
+
+      LLValue *arrayPtr = DtoArrayPtr(toElem(e->e1));
+      Type *srcElementType = tsrc->nextOf();
+
+      if (DtoMemType(elementType) == DtoMemType(srcElementType)) {
+        DtoMemCpy(dstMem, arrayPtr);
+      } else {
+        for (unsigned i = 0; i < N; ++i) {
+          DLValue srcElement(srcElementType, DtoGEP1(arrayPtr, i));
+          LLValue *llVal = getCastElement(&srcElement);
+          DtoStore(llVal, DtoGEP(dstMem, 0, i));
+        }
+      }
     } else {
+      // Try a splat vector constant, otherwise store element-wise.
       Logger::println("normal (splat) expression");
       DValue *val = toElem(e->e1);
-      LLValue *llElement = DtoRVal(DtoCast(e->loc, val, elementType));
+      LLValue *llElement = getCastElement(val);
       if (auto llConstant = isaConstant(llElement)) {
-        auto vectorConstant = llvm::ConstantVector::getSplat(N, llConstant);
+#if LDC_LLVM_VER >= 1100
+        const auto elementCount = llvm::ElementCount(N, false);
+#else
+        const auto elementCount = N;
+#endif
+        auto vectorConstant =
+            llvm::ConstantVector::getSplat(elementCount, llConstant);
         DtoStore(vectorConstant, dstMem);
       } else {
         for (unsigned int i = 0; i < N; ++i) {
@@ -2655,6 +2647,14 @@ public:
     result = emitVector(e, vector);
   }
 
+  void visit(VectorArrayExp* e) override {
+    IF_LOG Logger::print("VectorArrayExp::toElem() %s\n", e->toChars());
+    LOG_SCOPE;
+
+    DValue *vector = toElem(e->e1);
+    result = DtoCastVector(e->loc, vector, e->type);
+  }
+
   //////////////////////////////////////////////////////////////////////////////
 
   void visit(PowExp *e) override {
@@ -2824,7 +2824,6 @@ bool toInPlaceConstruction(DLValue *lhs, Expression *rhs) {
           auto lval = DtoLVal(lhs);
           ToElemVisitor::emitStructLiteral(sle, lval);
           // ... and invoke the ctor directly on it
-          DtoDeclareFunction(fd);
           auto fnval = new DFuncValue(fd, DtoCallee(fd), lval);
           DtoCallFunction(ce->loc, ce->type, fnval, ce->arguments);
           return true;
@@ -2886,14 +2885,3 @@ bool toInPlaceConstruction(DLValue *lhs, Expression *rhs) {
 
   return false;
 }
-
-////////////////////////////////////////////////////////////////////////////////
-
-// FIXME: Implement & place in right module
-Symbol *toModuleAssert(Module *m) { return nullptr; }
-
-// FIXME: Implement & place in right module
-Symbol *toModuleUnittest(Module *m) { return nullptr; }
-
-// FIXME: Implement & place in right module
-Symbol *toModuleArray(Module *m) { return nullptr; }
