@@ -30,6 +30,21 @@ IrTypeStruct::IrTypeStruct(StructDeclaration *sd)
 
 //////////////////////////////////////////////////////////////////////////////
 
+std::vector<IrTypeStruct*> IrTypeStruct::dcomputeTypes;
+
+/// Resets special DCompute structs so they get re-created
+/// with the proper address space when generating device code.
+void IrTypeStruct::resetDComputeTypes() {
+  for(auto&& irTypeStruct : dcomputeTypes) {
+    delete irTypeStruct->dtype->ctype;
+    irTypeStruct->dtype->ctype = nullptr;
+  }
+
+  dcomputeTypes.clear();
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
 IrTypeStruct *IrTypeStruct::get(StructDeclaration *sd) {
   auto t = new IrTypeStruct(sd);
   sd->type->ctype = t;
@@ -45,9 +60,14 @@ IrTypeStruct *IrTypeStruct::get(StructDeclaration *sd) {
 
   t->packed = isPacked(sd);
 
+  if(isFromLDC_DCompute(sd)) {
+    dcomputeTypes.push_back(t);
+  }
+  
   // For ldc.dcomptetypes.Pointer!(uint n,T),
   // emit { T addrspace(gIR->dcomputetarget->mapping[n])* }
-    llvm::Optional<DcomputePointer> p;
+  llvm::Optional<DcomputePointer> p;
+
   if (gIR->dcomputetarget && (p = toDcomputePointer(sd))) {
    
     // Translate the virtual dcompute address space into the real one for
