@@ -1,5 +1,7 @@
 #!/usr/bin/env bash
 
+source tools/common_funcs.sh
+
 expect_file=${EXTRA_FILES}/${TEST_NAME}.out
 obj_file=${OUTPUT_BASE}_0.o
 
@@ -12,4 +14,4 @@ echo SANITIZING Objdump...
 
 diff -pu --strip-trailing-cr "${expect_file}" "${obj_file}.dump.sanitized"
 
-rm -f "${OUTPUT_BASE}.o"{,.dump,.dump,.sanitized}
+rm_retry "${OUTPUT_BASE}.o"{,.dump,.dump,.sanitized}
