@@ -6,4 +6,4 @@ echo 'import std.stdio; void main() { writeln("Success"); }' | \
 	$DMD -m${MODEL} -run - > ${output_file}
 grep -q 'Success' ${output_file}
 
-rm "${output_file}"
+rm_retry "${output_file}"
